@@ -1,48 +1,86 @@
 (* 
- * $Id: unixqueue.mli 459 2006-04-30 19:49:19Z gerd $
+ * $Id: unixqueue.mli 1616 2011-06-10 15:08:57Z gerd $
  *)
 
 open Unix;;
 open Sys;;
 
-(**********************************************************************)
-(*                          System events                             *)
-(**********************************************************************)
+(** Unixqueues are one of the two forms of system event loops provided
+    by Ocamlnet. Besides Unixqueue, there is also pollset (see
+    {!Netsys_pollset}). The pollsets are much simpler (there is no
+    queuing of events), and nowadays Unixqueue bases upon pollset,
+    and extends its functionality. Historically, however, Unixqueue
+    precede pollset, and there are still implementations of the former
+    in Ocamlnet not using pollset as its base data structure.
 
-(* written by Gerd Stolpmann *)
+    The common idea of both data structures is the generalization of 
+    watching for events, as it is also provided by the [Unix.select]
+    function. Note, however, that recent implementations no longer
+    use [Unix.select], but better system interfaces for the same.
 
-(** This module generalizes the [Unix.select] function. The idea is to have
- * an event queue (implemented by {!Equeue}) that manages all file events that
- * can be watched by a [Unix.select] call. As {b event} is considered when there
- * is something to do for a file descriptor (reading, writing, accepting 
- * out-of-band data), but also the condition that for a certain period 
- * of time ("timeout") nothing
- * has happened. Furthermore, a signal is also considered as an event,
- * and it is also possible to have user-generated extra events.
- *
- * These events are queued up, and they are presented to event handlers
- * that may process them.
- *
- * You can describe what types of event conditions are watched by adding
- * "resources". You can think a resource being a condition (bound to
- * a real resource of the operating system) for which 
- * events are generated if the condition becomes true.
+    When there is something to do for a file descriptor (reading, 
+    writing, accepting out-of-band data), this is called an {b event},
+    and the task of Unixqueue is to check when events happen, and to
+    tell some consumer about the events.
+
+    There are three further types of events: Timeout events, signal
+    events, and user-defined events.
+
+    The events are queued up, and they are presented to event handlers
+    that may process them.
+
+    You can describe what types of event conditions are watched by adding
+    {b resources}. You can think a resource being a condition (bound to
+    a real resource of the operating system) for which 
+    events are generated if the condition becomes true. Currently, only
+    file descriptors and timers are supported as resources.
  *)
 
-(** {b THREAD-SAFETY}
- * 
- * Since release 1.2 of Equeue, this module serializes automatically.
- * You can call functions for the same event system from different
- * threads. This requires some special initialization, see {!Unixqueue_mt}.
- *
- * Note that the underlying {!Equeue} module is reentrant, but not
- * serializing. (It is not recommended (and not necessary) to call
- * functions of the Equeue module directly in multi-threaded programs.)
- *
- * The TCL extension is not thread-safe.
+(** {b Relation to other modules.} This module is thought as the primary
+    interface to Unixqueues. If there isn't any specialty one has to deal
+    with, just use this module:
+
+    - It defines all required types like [group], [wait_id], etc. Note that
+      these types are reexported from [Unixqueue_util]. Please consider
+      this as implementation detail, and don't use it in your code.
+    - It defines a standard implementation [standard_event_system], which
+      is a good default implementation, although it might not be the best
+      available for all purposes.
+    - It defines a set of access functions like [add_event] which 
+      simply call the methods of the event system object of the same name.
+      Note that these functions work for all event system implementation,
+      not only for [standard_event_system].
+
+    There are further modules that have to do with Unixqueue:
+
+    - {!Unixqueue_pollset} is the implementation behind 
+      [standard_event_system]. If you want to use other pollsets than
+      the standard one, it is possible to create Unixqueues on top of these
+      by using this module directly.
+    - {!Unixqueue_select} is the historic default implementation. It
+      calls directly [Unix.select]. It is still available because it 
+      serves as a reference implementation for now.
+    - [Unixqueue_util] is an internal module with implementation details.
+      Please don't call it directly.
+    - {!Uq_gtk} is an implementation of Unixqueue mapping to the 
+      GTK event loop. Useful for multiplexing event-based I/O and
+      GTK graphics operations.
+    - {!Uq_tcl} is an implementation of Unixqueue mapping to the 
+      TCL event loop. Useful for multiplexing event-based I/O and
+      event-based code written in TCL (especially TK).
  *)
 
-type group
+(** {b Thread safety.} The default implementation of Unixqueue is
+    thread-safe, and operations can be called from different threads.
+    For other implementations, please look at the modules implementing
+    them.
+ **)
+
+
+(** {1 Types and exceptions} *)
+
+
+type group = Unixqueue_util.group
   (** A group is an abstract tag for a set of events, resources, and
    * event handlers. Usually every event handler creates a new group,
    * and all events and resources processed by the handler are 
@@ -58,16 +96,20 @@ exception Abort of (group * exn);;
    * First argument is the group. The second argument
    * is an arbitrary exception (must not be [Abort] again) which is
    * passed to the abort action.
+   *
+   * Abort handlers are a questionable feature of Unixqueues. You
+   * can also call the [clear] operation, and raise the exception
+   * directly. {b Do not use in new code!}
    *)
 
 
-type wait_id
+type wait_id = Unixqueue_util.wait_id
   (** A wait identifier is used to distinguish between several
    * timers, see type [operation].
    *)
 
 
-type operation =
+type operation = Unixqueue_util.operation =
     Wait_in  of file_descr          (** wait for input data *)
   | Wait_out of file_descr          (** wait until output can be written *)
   | Wait_oob of file_descr          (** wait for out-of-band data *)
@@ -77,13 +119,14 @@ type operation =
    *)
 
 
-type event =
+type event = Unixqueue_util.event =
     Input_arrived of    (group * file_descr)  (** Input data has arrived *)
   | Output_readiness of (group * file_descr)  (** Output is possible now *)
   | Out_of_band of      (group * file_descr)  (** OOB data has arrived *)
   | Timeout of          (group * operation)   (** A timer has expired *)
   | Signal                                    (** A signal has happened *)
   | Extra of exn                              (** User-generated event *)
+  | Immediate of (group * (unit -> unit))     (** immediate event *)
   (** An [event] is triggered when the condition of an [operation]
    * becomes true, when a signal happens, or when the event is
    * (artificially) added to the event queue ([add_event], below).
@@ -103,6 +146,14 @@ type event =
    * The event [Extra] can only be artificially added to the queue,
    * and the argument of [Extra] is an exception value that distinguishes
    * between several kinds of user-generated events.
+   *
+   * The event [Immediate(g,f)] also can only be artificially added to
+   * the queue. In contrast to other events, it is not passed to handlers
+   * when the event is processed. Instead, an immediate event is processed
+   * by calling [f()]. This is a more direct way of notification, and
+   * it is not necessary to define a handler. Even an immediate event is
+   * member of a group [g], and if the [clear] function is called for [g],
+   * the callback function [f] will no longer be called.
    *)
 
 class type event_system =
@@ -112,6 +163,7 @@ object
   method new_wait_id : unit -> wait_id
   method exists_resource : operation -> bool
   method add_resource : group -> (operation * float) -> unit
+  method add_weak_resource : group -> (operation * float) -> unit
   method add_close_action : group -> (file_descr * (file_descr -> unit)) -> unit
   method add_abort_action : group -> (group -> exn -> unit) -> unit
   method remove_resource : group -> operation -> unit
@@ -120,13 +172,7 @@ object
   method clear : group -> unit
   method run : unit -> unit
   method is_running : bool
-  method once : group -> float -> (unit -> unit) -> unit
-  method exn_log : ?suppressed:bool -> ?to_string:(exn -> string) -> ?label:string -> exn -> unit
-  method debug_log : ?label:string -> string -> unit
-  (* Protected interface *)
-  method private setup : unit -> (file_descr list * file_descr list * file_descr list * float)
-  method private queue_events : (file_descr list * file_descr list * file_descr list) -> bool
-  method private source : event Equeue.t -> unit
+  method when_blocking : (unit -> unit) -> unit
 end
   (** The [event_system] manages events, handlers, resources, groups,
    * etc. It is now a class type, and you may invoke the operations directly
@@ -180,11 +226,34 @@ end
    * 
    *)
 
+(** {1 Creating event systems} *)
+
+class standard_event_system : unit -> event_system
+  (** The standard implementation of an event system. It uses 
+      {!Unixqueue_pollset.pollset_event_system} on top of
+      {!Netsys_pollset_generic.standard_pollset}.
+   *)
+
+val standard_event_system : unit -> event_system
+  (** Create a new, empty, standard event system *)
+
 class unix_event_system : unit -> event_system
-  (** The standalone implementation of an event system *)
+  (** An alternate name for [standard_event_system], provided for
+      backward compatibility.
+   *)
 
 val create_unix_event_system : unit -> event_system
-  (** Create a new, empty event system *)
+  (** An alternate name for [standard_event_system], provided for
+      backward compatibility.
+   *)
+
+
+(** {1 Using event systems} *)
+
+(** The following functions work for all kinds of event systems, not
+    only for the ones returned by [standard_event_system].
+ *)
+
 
 val new_group : event_system -> group
   (** Create a new, empty group for the event system *)
@@ -213,6 +282,20 @@ val add_resource : event_system -> group -> (operation * float) -> unit
    * period starts again in this case.
    *)
 
+val add_weak_resource : event_system -> group -> (operation * float) -> unit
+  (** Similar to [add_resource], but the resource is weak. Such resources
+    * do not keep the event system running when only weak resources remain.
+    * Normally, [Unixqueue.run] returns to the caller not before
+    * all resources are removed and all events are processed. Weak
+    * resources do not count for this condition, i.e. [Unixqueue.run]
+    * also returns when there are only weak resources left.
+    * As an example, weak resources can be used to time out unused
+    * file descriptors.
+    *
+    * Weak resources can be removed with [remove_resource].
+    *
+    * {b New in Ocamlnet 3.}
+   *)
 
 val add_close_action : 
   event_system -> group -> (file_descr * (file_descr -> unit)) 
@@ -228,6 +311,16 @@ val add_close_action :
    * is called.
    *
    * You can only add (set) one close action for every descriptor.
+   *
+   * Of course, the idea is to do [add_close_action ... Unix.close]. Note
+   * that there is a problem with multi-threaded programs, and this construct
+   * must not be used there. In particular, the close action is called from
+   * [remove_resource] or [clear], but it is possible that the event system
+   * is running, so a watched descriptor might be closed. This has undesired
+   * effects. What you should better do is to delay the closure of the
+   * descriptor to a sane moment, e.g. by calling
+   *   {[ Unixqueue.once esys g 0.0 (fun () -> Unix.close fd) ]}
+   * from the close action.
    *)
 
 
@@ -312,9 +405,45 @@ val once : event_system -> group -> float -> (unit -> unit)
    * group, the timer is deleted, too.
    *)
 
-val exn_log : event_system ->
+val weak_once : event_system -> group -> float -> (unit -> unit) -> unit
+  (** Same as [once], but the timer does not keep the event system running
+      if it is the only remaining resource.
+   *)
+
+val epsilon : event_system -> (unit -> unit) -> unit
+  (** The execution of the function is pushed onto the event queue
+      (minimal delay)
+   *)
+
+
+(** {1 Debugging} *)
+
+
+module Debug : sig
+  (** This module controls debugging of all [Unixqueue_*] modules *)
+
+  val enable : bool ref
+    (** Enable {!Netlog} debugging *)
+
+  val set_debug_mode : bool -> unit
+    (** Sets [enable].
+     *)
+
+  val set_debug_target : Equeue.Debug.debug_target -> unit
+    (** Restricts debugging to this target. 
+     *)
+end
+
+(*
+(** The status of the following functions is currently a bit unclear.
+    When a Ocamlnet-wide logging concept is introduced, these functions
+    probably disappear. They are used by other Ocamlnet modules to
+    implement debug logging.
+ *)
+
+(* val exn_log : event_system ->
               ?suppressed:bool -> ?to_string:(exn -> string) -> 
-              ?label:string -> exn -> unit
+              ?label:string -> exn -> unit *)
   (** Exceptions log: In event-based programming, it is sometimes not
    * possible to handle exceptions appropriately. It is also bad not
    * to handle them at all. For these cases, the exceptions log might
@@ -351,14 +480,14 @@ val exn_log : event_system ->
    *   useful to describe where the log message is generated.
    *)
 
-val debug_log : event_system -> ?label:string -> string -> unit
+(* val debug_log : event_system -> ?label:string -> string -> unit*)
   (** Outputs a message in the debug log (when enabled).
    *
    * @param label The label is included in the log output. This is
    *   useful to describe where the log message is generated.
    *)
 
-val set_debug_mode : bool -> unit
+(* val set_debug_mode : bool -> unit *)
   (** Whether to output debug messages. Output goes to stderr.
    * Setting the debug mode implies setting Equeue's debug mode.
    *
@@ -367,14 +496,6 @@ val set_debug_mode : bool -> unit
    * is required to interpret debug protocols.
    *)
 
-(* val attach_to_tcl_queue : event_system -> (event_system -> unit) -> unit
- *
- * This is no longer supported here. Look into Uq_tcl for the modern
- * way to integrate with the tcl/tk event system
+(* val set_debug_target : Equeue.debug_target -> unit *)
+  (** More fine-grained debug control *)
  *)
-
-(**/**)
-
-val init_mt : (unit -> ((unit -> unit) * (unit -> unit))) -> unit
-(* A private function to initialize multi-threading. *)
-
