@@ -53,14 +53,7 @@ struct
   }
 
   let default_config =
-    let default_tmp_directory =
-      let candidates =
-	match Sys.os_type with
-	| "Unix" | "Cygwin" -> [ "/var/tmp"; "/tmp"; "." ]
-	| "Win32" -> [ "C:\\TEMP"; "." ]
-	| "MacOS" -> [ Filename.current_dir_name ]
-	| _ -> assert false in
-      List.find Sys.file_exists candidates in
+    let default_tmp_directory = Netsys_tmp.tmp_directory() in
     {
       tmp_directory = default_tmp_directory;
       tmp_prefix = "netcgi";
@@ -384,7 +377,7 @@ struct
     method finalize = arg#finalize
   end
 
-  type cgi_cookie = Nethttp.cookie = {
+  type cgi_cookie = Nethttp.netscape_cookie = {
     cookie_name : string;
     cookie_value : string;
     cookie_expires : float option;
@@ -581,7 +574,7 @@ struct
 
     method set_header ?status ?content_type ?content_length
       ?(set_cookie=[]) ?(set_cookies=[])
-      ?cache ?filename ?language ?script_type ?style_type ?fields
+      ?cache ?filename ?language ?script_type ?style_type ?(fields=[])
       () =
       let now = Unix.time() in
       let old_cookie c =
@@ -594,9 +587,14 @@ struct
 	  cookie_path = Netcgi_common.Cookie.path c;
 	  cookie_secure = Netcgi_common.Cookie.secure c;
 	} in
-      let fields = match content_length with
-	| None -> []
-	| Some size -> [ "content-length", [string_of_int size] ] in
+      let fields = 
+	match content_length with
+	  | None -> fields
+	  | Some size -> 
+	      ("content-length", [string_of_int size]) ::
+		(List.filter
+		   (fun (n,_) -> String.lowercase n <> "content-length")
+		   fields) in
       (* The old [set_header] knows whether the output is
 	 transactional or not. *)
       cgi_act#set_header ?status ?content_type ?cache ?filename
