@@ -1,4 +1,4 @@
-(* $Id: http_client.mli 945 2006-07-11 21:35:50Z gerd $ *)
+(* $Id: http_client.mli 1661 2011-08-28 22:45:55Z gerd $ *)
 
 (** HTTP 1.1 client *)
 
@@ -11,27 +11,41 @@
   * {!Http_client.Convenience}.
  *)
 
-(** Implements much of HTTP/1.1.
+(** 
  * Implements the following advanced features:
- *  - chunked messages
+ *  - chunked message transport
  *  - persistent connections
  *  - connections in pipelining mode ("full duplex" connections)
  *  - modular authentication methods, currently Basic and Digest
  *  - event-driven implementation; allows concurrent service for
  *    several network connections 
+ *  - HTTP proxy support, also with Basic and Digest
+ *    authentication
+ *  - SOCKS proxy support (1)
+ *  - HTTPS support via extension module {!Https_client} (in library
+ *    [equeue-ssl]). HTTPS proxies are also supported (CONNECT method) (1)
+ *  - Automatic and configurable retry of failed idempotent requests
+ *  - Redirections can be followed automatically 
+ *  - Compressed message bodies can be automatically decoded (gzip only,
+ *    method [set_accept_encoding]) (1)
  *
  * Left out:
  *  - multipart messages, including multipart/byterange
- *  - content encoding (compression)    (1)
- *  - content digests specified by RFC 2068 and 2069   (1)
- *  - content negotiation   (1)
- *  - conditional and partial GET   (1)
- *  - following code 303 redirections automatically    (1)
- *  - client-side caching   (1)
+ *  - content digests specified by RFC 2068 and 2069   (2)
+ *  - content negotiation   (2)
+ *  - conditional and partial GET   (2)
+ *  - client-side caching   (2)
  *  - HTTP/0.9 compatibility
  *
- * (1) These features can be implemented on top of this module if really needed,
+ * (1) Since Ocamlnet-3.3
+ *
+ * (2) These features can be implemented on top of this module if really needed,
  *     but there is no special support for them.
+ *
+ * Related modules/software:
+ * - {!Http_fs} allows you to access HTTP servers in the style of filesystems
+ * - WebDAV: If you are looking for WebDAV there is an extension of this module:
+ *   {{:http://oss.wink.com/webdav/} Webdav}, which is separately available.
  *)
 
 (** {b Thread safety}
@@ -67,12 +81,25 @@ exception Too_many_redirections
   (** While following redirections the limit has been reached *)
 
 exception Name_resolution_error of string
-  (** Could not resolve this name *)
+  (** Could not resolve this name - same as {!Uq_engines.Host_not_found} *)
 
 exception URL_syntax_error of string
   (** This URL cannot be parsed after a redirection has been followed.
    *)
 
+exception Timeout of string
+  (** A timeout. The string explains which connection is affected.
+      {b New since Ocamlnet-3.3.}
+   *)
+
+exception Proxy_error of int
+  (** An error status from a proxy. This is only used when extra proxy messages
+      are used to configure the proxy (e.g. the CONNECT message).
+   *)
+
+exception Response_too_large
+  (** The length of the response exceeds the configured maximum *)
+
 exception Http_protocol of exn;;
   (** The request could not be processed because the exception condition 
    * was raised. The inner exception is one of the above defined.
@@ -82,7 +109,8 @@ exception Http_error of (int * string);;
   (** {b Deprecated.} 
    * The server sent an error message. The left component of the pair is
    * the error code, the right component is the error text.
-   * This exception is only used by [get_resp_body].
+   * This exception is only used by [get_resp_body], and by the
+   * {!Http_client.Convenience} module.
    *)
 
 type status =
@@ -145,6 +173,7 @@ type response_body_storage =
     [ `Memory
     | `File of unit -> string
     | `Body of unit -> Netmime.mime_body
+    | `Device of unit -> Uq_io.out_device
     ]
   (** How to create the response body:
     * - [`Memory]: The response body is in-memory
@@ -152,6 +181,14 @@ type response_body_storage =
     *   is returned by [f()]
     * - [`Body f]: The response body is stored into the object returned
     *   by [f()]
+    * - [`Device f]: The response is directly forwarded to the device
+    *   obtained by [f()] (new since Ocamlnet-3.3)
+    *
+    * When the function [f] is called in the latter cases the response
+    * header has already been received, and can be retrieved with the
+    * [response_header] method of the call object. Also, [response_status_text],
+    * [response_status_code], and [response_status] return meaningful
+    * values.
    *)
 
 
@@ -203,6 +240,20 @@ type resolver =
           * Only 1:1 resolution is supported, 1:n resolution not.
          *)
 
+type channel_binding_id = int
+    (** A channel binding identifies a requirement for the transport
+	channel, especially whether plain HTTP is sufficient, or HTTPS
+	needs to be used, and if so, whether there are further requirements
+	for the SSL context. There are the predefined IDs:
+
+	- {!Http_client.http_cb_id} for HTTP connections
+	- {!Http_client.https_cb_id} for HTTPS connections without user
+	  certificates
+        - {!Http_client.proxy_only_cb_id} for the restriction that this
+	  protocol can only be used via a web proxy
+     *)
+
+
 type http_options = 
     { synchronization : synchronization;
         (** Default: [Pipeline 5]. *)
@@ -246,12 +297,19 @@ type http_options =
         (** The function for name resolution *)
       configure_socket : Unix.file_descr -> unit;
         (** A function to configure socket options *)
+      schemes : (string * Neturl.url_syntax * int option * channel_binding_id) 
+                   list;
+        (** The list of supported URL schemes. The tuples mean
+	    [(scheme, syntax, default_port, cb)]. By default, the
+	    schemes "http", "https", and "ipp" are supported.
+	 *)
       verbose_status : bool;
       verbose_request_header : bool;
       verbose_response_header : bool;
       verbose_request_contents : bool;
       verbose_response_contents : bool;
       verbose_connection : bool;
+      verbose_events : bool;
         (** Enable various debugging message types.
 	 * - [verbose_status]: reports about status of received documents
 	 * - [verbose_request_header]: prints the header sent to the server
@@ -260,9 +318,24 @@ type http_options =
 	 * - [verbose_response_contents]: prints the document received from the server
 	 * - [verbose_connection]: reports many connection events; authentication,
 	 *   too.
+         * - [verbose_events]: everything about the interaction with [Unixqueue]
+         *
+         * By default, [verbose_status] and [verbose_connection] are enabled.
+         * Note that you also have to set [Debug.enable] to [true] to see
+         * any log message at all!
 	 *)
     }
-  (** Options for the whole pipeline *)
+      (** Options for the whole pipeline. It is recommended to change options
+	* the following way:
+	* {[
+	*    let opts = pipeline # get_options in
+        *    let new_opts = { opts with <field> = <value>; ... } in
+        *    pipeline # set_options new_opts
+        * ]}
+        *
+        * New fields can be added anytime to this record, and this style
+        * of changing options is transparent to field additions.
+       *)
 ;;
 
 
@@ -277,30 +350,30 @@ type header_kind = [ `Base | `Effective ]
 class type http_call =
 object
   (** [http_call] is the runtime container for HTTP method calls. It contains
-    * the request message, the response message, and the current transmission
-    * status.
-    *
-    * In previous versions of netclient, this class type was called 
-    * [message]. This was quite inexact because this class embraces both
-    * messages that are part of a call.
-    *
-    * {b Incompatible changes}:
-    * - [using_proxy] has been removed. This simply isn't a property of
-    *   an individual call.
-    * - [get_req_uri] has been removed from the public interface for similar
-    *   reasons.
-    * - The request and response messages are now primarily stored as
-    *   [Netmime.mime_header] and [Netmime.mime_body] objects. The old
-    *   style access methods remain in the API for now. The advantage is
-    *   that it is possible to use external files as body containers.
-    * - There are two request headers, [`Base] and [`Effective].
+      * the request message, the response message, and the current transmission
+      * status.
+      *
+      * In previous versions of netclient, this class type was called 
+      * [message]. This was quite inexact because this class embraces both
+      * messages that are part of a call.
+      *
+      * {b Incompatible changes}:
+      * - [using_proxy] has been removed. This simply isn't a property of
+      *   an individual call.
+      * - [get_req_uri] has been removed from the public interface for similar
+      *   reasons.
+      * - The request and response messages are now primarily stored as
+      *   [Netmime.mime_header] and [Netmime.mime_body] objects. The old
+      *   style access methods remain in the API for now. The advantage is
+      *   that it is possible to use external files as body containers.
+      * - There are two request headers, [`Base] and [`Effective].
    *)
 
   (** {2 Call State} *)
 
   method is_served : bool
     (** [true] if request/response cycle(s) have been finished, i.e. the
-      * call was successful, or a final error state has been reached.
+	* call was successful, or a final error state has been reached.
      *)
 
   method status : status
@@ -313,52 +386,99 @@ object
 
   method request_uri : string
     (** The request URI as string. This is always an absolute URI in the
-      * form "http://server/path".
+	* form "http://server/path" or "https://server/path".
      *)
 
   method set_request_uri : string -> unit
-    (** Sets the request URI *)
+    (** Sets the request URI. This implicitly also sets the channel binding
+	ID (see below).	
+
+	{b Changed in Ocamlnet-3.3:} The URI is no longer immediately parsed,
+	but first when the call is submitted to a pipeline. This means that
+	parsing errors will first be reported by the [add] method. The
+	background is that parsing now depends on pipeline options.
+     *)
 
   method request_header : header_kind -> Netmime.mime_header
     (** The whole header of the request. Users of this class should only
-     * modify the [`Base] header. After the call has been processed,
-     * the [`Effective] header contains the version of the header that has
-     * actually been transmitted.
-     *
-     * The user should set the following headers:
-     * - [Content-length]: Set this to the length of the request body
-     *   if known. (The client falls back to HTTP 1.0 if not set!)
-     * - [Content-type]: Set this to the media type of the request body
-     * - [Expect]: Set this to "100-continue" to enable a handshake before
-     *   the body is sent. Recommended for large bodies.
-     *
-     * The following headers may be set, but there are reasonable defaults:
-     * - [Date]
-     * - [User-agent]
-     *
-     * The following headers must not be set:
-     * - [Connection]
+	* modify the [`Base] header. After the call has been processed,
+	* the [`Effective] header contains the version of the header that has
+	* actually been transmitted.
+	*
+	* The user should set the following headers:
+	* - [Content-length]: Set this to the length of the request body
+	*   if known. (The client falls back to HTTP 1.0 if not set!)
+	* - [Content-type]: Set this to the media type of the request body
+	* - [Expect]: Set this to "100-continue" to enable a handshake before
+	*   the body is sent. Recommended for large bodies. (See also
+	*   [set_expect_handshake] below.)
+	*
+	* The following headers may be set, but there are reasonable defaults:
+	* - [Date]
+	* - [User-agent]
+	*
+	* The following headers must not be set:
+	* - [Connection]
      *)
 
   method set_request_header : Netmime.mime_header -> unit
     (** Replaces the whole [`Base] header object *)
 
+  method set_expect_handshake : unit -> unit
+    (** Configures that a handshake is done before sending the request body.
+	This is useful when the request body can be large, and
+	authentication or response indirections are possible.
+	{b New since Ocamlnet-3.3.}
+     *)
+
+  method set_chunked_request : unit -> unit
+    (** Configures that the request is transmitted using chunked encoding.
+	This has advantages when the size of the request is not known
+	in advance. However, this works only for true HTTP/1.1 servers.
+	{b New since Ocamlnet-3.3.}
+     *)
+
   method effective_request_uri : string
     (** The URI actually sent to the server in the request line of the
-      * protocol.
+	* protocol.
      *)
 
   method request_body : Netmime.mime_body
-    (** The whole body of the request *)
+    (** The whole body of the request. This method fails after
+	[set_request_device] has been called (no body then).
+     *)
 
   method set_request_body : Netmime.mime_body -> unit
     (** Replaces the whole body object *)
 
+  method set_request_device : (unit -> Uq_io.in_device) -> unit
+    (** Sets that the request data is read from the passed device instead
+	of taking it from a body object.  The device must be connected with
+	the same event loop as the HTTP client.
+     *)
+
+  method set_accept_encoding : unit -> unit
+    (** Sets the [Accept-Encoding] field in the request header, and 
+	includes all decompression algorithms registered in
+	{!Netcompression}. Additionally, the automatic decompression of
+	the response body is enabled.
+
+	Note that you need to ensure that the algorithms are really
+	registered at {!Netcompression}. For example, to get [gzip]
+	support, run {[ Netgzip.init() ]}, and include [netzip] as
+	library.
+     *)
+
+
   (** {2 Accessing the response message (new style) }
-    *
-    * These method will fail if the call has not yet been served! 
-    * If the call has been finished, but was not successful, the
-    * exception [Http_protocol] is raised.
+      *
+      * These methods will fail if the call has not yet been served! 
+      * If the call has been finished, but a hard error (e.g. socket error)
+      * occurred, the
+      * exception [Http_protocol] is raised. When the server only
+      * sent an error code, no exception is raised - but the user can
+      * manually test for such codes (e.g. with [repsonse_status] or
+      * [status]).
    *)
 
   method response_status_code : int
@@ -369,8 +489,8 @@ object
 
   method response_status : Nethttp.http_status
     (** The decoded code. Unknown codes are mapped to the generic status
-      * values [`Ok], [`Multiple_choices], [`Bad_request], and 
-      * [`Internal_server_error].
+	* values [`Ok], [`Multiple_choices], [`Bad_request], and 
+	* [`Internal_server_error].
      *)
 
   method response_protocol : string
@@ -378,13 +498,17 @@ object
 
   method response_header : Netmime.mime_header
     (** The whole header of the response. If the call has not succeeded, 
-      * [Http_protocol] will be raised.
+	* [Http_protocol] will be raised.
      *)
 
   method response_body : Netmime.mime_body
     (** The whole body of the response. If the call has not succeeded, 
-      * [Http_protocol] will be raised. If the call has succeeded, 
-      * but no body has been transmitted, the empty body is substituted.
+	* [Http_protocol] will be raised. If the call has succeeded, 
+	* but no body has been transmitted, the empty body is substituted.
+	*
+	* If the response is directly forwarded to a device (after
+	* calling [set_response_body_storage (`Device d)]), there is
+	* no accessible response body, and this method will fail.
      *)
 
 
@@ -396,10 +520,20 @@ object
   method set_response_body_storage : response_body_storage -> unit
     (** Sets how to create the response body *)
 
+  method max_response_body_length : int64
+    (** Returns the current maximum length (initially [Int64.max_int]) *)
+
+  method set_max_response_body_length : int64 -> unit
+    (** Sets a new maximum length. When the body exceeds this maximum
+	by more than the size of the internal buffer, the reception of
+	the response is interrupted, and the error is set to
+	[Response_too_large].
+     *)
+
   method get_reconnect_mode : http_call how_to_reconnect
     (** Get what to do if the server needs to be reconnected, i.e.
-      * if the request must be sent repeatedly.
-      * By default, this is [Send_again_if_idem].
+	* if the request must be sent repeatedly.
+	* By default, this is [Send_again_if_idem].
      *)
 
   method set_reconnect_mode : http_call how_to_reconnect -> unit
@@ -424,80 +558,125 @@ object
     (** {b Deprecated.} Same as [proxy_enabled] *)
 
   (** {2 Method characteristics}
-    *
-    * These properties describe the HTTP method 
-    *)
+      *
+      * These properties describe the HTTP method 
+   *)
+
+  method proxy_use_connect : bool
+    (** Whether to use the CONNECT method if the connection is made via a
+	web proxy. This is normally true if the channel binding is
+	{!Http_client.https_cb_id}
+     *)
 
   method empty_path_replacement : string
     (** The string to substitute in the request line for the empty
-      * path. This is usually "/", and for OPTIONS it is "*".
+	* path. This is usually "/", and for OPTIONS it is "*".
      *)
 
   method is_idempotent : bool
     (** Whether the method is to be considered as idempotent ( = repeated
-      * invocations have the same result and no side effect). This is
-      * true for GET and HEAD.
+	* invocations have the same result and no side effect). This is
+	* true for GET and HEAD.
      *)
 
   method has_req_body : bool
     (** Whether the method allows to send a request body *)
 
   method has_resp_body : bool
-      (** Whether the method allows to reply with a body. This is true
+    (** Whether the method allows to reply with a body. This is true
         * except for HEAD.
-       *)
+     *)
+
+  (** {2 Channel bindings} 
+
+      Channel bindings are used to distinguish between security requirements.
+      There are normally only two types of requirements:
+
+      - The ID {!Http_client.http_cb_id} is used for messages that can only
+        be sent over HTTP connections, i.e. unencrypted TCP. It is automatically
+        set when the URL of the message starts with "http://".
+      - The ID {!Http_client.https_cb_id} describes the requirement that the
+        message can only be sent over HTTPS connections, i.e. TLS-protected
+        TCP. It is automatically  set when the URL of the message starts with
+        "https://".
+
+      It is possible to change the channel binding to establish further
+      types of security requirements (e.g. that certain client certificates
+      are used), or even other details of the transport connection.
+
+      The method [channel_binding] is gone from this object type.
+      It is now available as part of [pipeline].
+   *)
+
+  method set_channel_binding : channel_binding_id -> unit
+    (** Sets the channel binding. Note that [set_request_uri] also sets
+	the channel binding, but always to the default for the type of URL.
+     *)
 
   (** {2 Repeating calls} *)
 
   method same_call : unit -> http_call
     (** This method returns a new object that will perform the same call
-      * as this object (this function is called "reload" in browsers).
-      * The new object is initialized as follows:
-      * - The state is set to [`Unserved]
-      * - The request method remains the same (the class of the returned
-      *   object remains the same)
-      * - The request URI is the same string as the original URI
-      * - The base request header is the same object
-      * - The request body is the same object
-      * - Options like reconnect, redirect mode, and proxy mode are
-      *   copied.
+	* as this object (this function is called "reload" in browsers).
+	* The new object is initialized as follows:
+	* - The state is set to [`Unserved]
+	* - The request method remains the same (the class of the returned
+	*   object remains the same)
+	* - The request URI is the same string as the original URI
+	* - The channel binding ID is the same
+	* - The base request header is the same object
+	* - The request body is the same object
+	* - Options like reconnect, redirect mode, and proxy mode are
+	*   copied.
      *)
 
   (** {2 Old style access methods}
-    *
-    * These method were introduced in previous versions of netclient,
-    * but are quite limited. Some questionable methods are now deprecated
-    * and will be removed in future versions of netclient.
+      *
+      * These method were introduced in previous versions of netclient,
+      * but are quite limited. Some questionable methods are now deprecated
+      * and will be removed in future versions of netclient.
    *)
 
   method get_req_method : unit -> string
     (** Get the name of the request method. Same as [request_method]. *)
 
   method get_host : unit -> string
-    (** The host name of the content server, extracted from the URI *)
+    (** The host name of the content server, extracted from the URI.
+	
+	{b Changed in Ocamlnet-3.3:} The host can first be extracted after
+	the call is submitted to a pipeline.
+     *)
 
   method get_port : unit -> int
-    (** The port number of the content server, extracted from the URI *)
+    (** The port number of the content server, extracted from the URI.
+	
+	{b Changed in Ocamlnet-3.3:} The port can first be extracted after
+	the call is submitted to a pipeline.
+     *)
 
   method get_path : unit -> string
-    (** The path extracted from the URI *)
+    (** The path (incl. query, if any) extracted from the URI.
+	
+	{b Changed in Ocamlnet-3.3:} The path can first be extracted after
+	the call is submitted to a pipeline.
+     *)
     
   method get_uri : unit -> string
     (** the full URI of this message: http://server:port/path. If the
-      * path is empty, it is omitted. - Same as [request_uri].
+	* path is empty, it is omitted. - Same as [request_uri].
      *)
 
   method get_req_body : unit -> string
     (** What has been sent as body in the (last) request. Same as
-      * [request_body # value].
+	* [request_body # value].
      *)
 
   method get_req_header : unit -> (string * string) list
     (** {b Deprecated.}
-      * What has been sent as header in the (last) request. Returns
-      * (key, value) pairs, where the keys are all in lowercase.
-      *
-      * In new code, the [request_header] object should be accessed instead.
+	* What has been sent as header in the (last) request. Returns
+	* (key, value) pairs, where the keys are all in lowercase.
+	*
+	* In new code, the [request_header] object should be accessed instead.
      *)
 
   method assoc_req_header : string -> string
@@ -505,8 +684,8 @@ object
 
   method assoc_multi_req_header : string -> string list
     (** Return all header values for a given field name (header entries
-      * which allow several values separated by commas can also be 
-      * transmitted by several header lines with the same name).
+	* which allow several values separated by commas can also be 
+	* transmitted by several header lines with the same name).
      *)
 
   method set_req_header : string -> string -> unit
@@ -514,8 +693,8 @@ object
 
   method get_resp_header : unit -> (string * string) list
     (** {b Deprecated.}
-      * Get the header of the last response. The keys are in lowercase 
-      * characters again.
+	* Get the header of the last response. The keys are in lowercase 
+	* characters again.
      *)
 
   method assoc_resp_header : string -> string
@@ -523,24 +702,24 @@ object
 
   method assoc_multi_resp_header : string -> string list
     (** Return all response header values for a given field name (header 
-      * entries which allow several values separated by commas can also be 
-      * transmitted by several header lines with the same name).
+	* entries which allow several values separated by commas can also be 
+	* transmitted by several header lines with the same name).
      *)
 
   method get_resp_body : unit -> string
     (** {b Deprecated.}
-     * Returns the body of the last response if the response status
-     * is OK (i.e. the code is in the range 200 to 299).
-     *
-     * Otherwise, Http_error (code, body) is raised where 'code' is
-     * the response code and 'body' is the body of the (errorneous)
-     * response.
+	* Returns the body of the last response if the response status
+	* is OK (i.e. the code is in the range 200 to 299).
+	*
+	* Otherwise, Http_error (code, body) is raised where 'code' is
+	* the response code and 'body' is the body of the (errorneous)
+	* response.
      *)
 
   method dest_status : unit -> (string * int * string)
     (** Returns the status line of the last response (but status lines
-     * with code 100 are ignored).
-     * The returned triple is (http_string, code, text)
+	* with code 100 are ignored).
+	* The returned triple is (http_string, code, text)
      *)
 
   (** {2 Private} *)
@@ -550,6 +729,34 @@ end
 ;;
 
 
+class type transport_channel_type =
+object
+  method setup_e : Unix.file_descr -> channel_binding_id -> float -> exn ->
+                   string -> int -> Unixqueue.event_system ->
+                   Uq_engines.multiplex_controller Uq_engines.engine
+  (** [setup fd cb tmo tmo_x host port esys]: Create or configure a communication
+      circuit over the file descriptor [fd] that can be driven by the
+      returned multiplex controller object.
+
+      [tmo] is the timeout. After inactivity the  exception [tmo_x] must be
+      raised.
+
+      [host] is the name of the machine to connect to. [port] is the port
+      number. The descriptor [fd] is already connected to this port, directly
+      or via a proxy.
+   *)
+
+  method continue : Unix.file_descr -> channel_binding_id -> float -> exn ->
+                   string -> int -> Unixqueue.event_system ->
+                   Uq_engines.multiplex_controller
+  (** [continue] is called when an already established circuit needs to
+      be continued.
+
+      Note that the event system can be different now.
+   *)
+end
+
+
 (** {1 HTTP methods} *)
 
 
@@ -638,11 +845,17 @@ object
     (** The realm *)
   method domain : string list
     (** The domain URIs defining the protection space. The domain URIs
-     * are absolute URIs. The list must not be empty.
+     * are absolute URIs. The list must not be empty for content accesses.
+     * For proxy keys the list must be empty.
      *)
 end
 
 
+val key : user:string -> password:string -> realm:string -> 
+          domain:string list -> key
+  (** Create a key object *)
+
+
 class type key_handler =
 object
   method inquire_key :
@@ -685,8 +898,12 @@ class type auth_session =
 object
   method auth_scheme : string
     (** The authentication scheme, e.g. "basic" *)
-  method auth_domain : string list
-    (** The list of domain URIs defines the protection space. *)
+  method auth_domain : Neturl.url list
+    (** The list of domain URIs defines the protection space.
+
+	{b Change:} Since Ocamlnet-3.3, this is a list of {!Neturl.url},
+	and no longer a list of strings.
+     *)
   method auth_realm : string
     (** The realm *)
   method auth_user : string
@@ -724,9 +941,11 @@ end
  *)
 class type auth_handler =
 object
-  method create_session : http_call -> auth_session option
+  method create_session : http_call -> http_options ref -> auth_session option
     (** Create a new authentication session. The passed call has status 401.
      *)
+  method create_proxy_session : http_call -> http_options ref -> auth_session option
+    (** Same for proxy authentication *)
 end
 
 class basic_auth_handler : 
@@ -759,6 +978,14 @@ class digest_auth_handler :
     * may be an additional security risk.
    *)
 
+class unified_auth_handler : #key_handler -> auth_handler
+  (** Support both digest and basic authentication, with preference to
+      digest.
+
+      Note that there is no way of authenticating in advance, as it is
+      not known in advance which mechanism is used.
+   *)
+
 
 (** {b Deprecated.} For (limited) backwards compatibility: *)
 class basic_auth_method :
@@ -775,14 +1002,14 @@ class basic_auth_method :
 class digest_auth_method : basic_auth_method
 
 
-(** {1 Pipelines} *)
+(** {1 Transport} *)
 
 (** A connection cache is an object that keeps connections open that
   * are currently unused. A connection cache can be shared by several
   * pipelines.
  *)
 
-type connection_cache
+type connection_cache = Http_client_conncache.connection_cache
 
 val close_connection_cache : connection_cache -> unit
   (** Closes all descriptors known to the cache *)
@@ -797,11 +1024,34 @@ val create_aggressive_cache : unit -> connection_cache
     * possible. The consequence is that users are responsible for
     * closing the descriptors (by calling [close_connection_cache]) when the
     * cache is no longer in use.
-    *
-    * Aggressive caching is an experimental feature. Reports about success
-    * and/or problems are very welcome.
    *)
 
+val http_cb_id : channel_binding_id
+  (** Identifies a channel binding to pure HTTP (without SSL), with or
+      without web proxies
+   *)
+
+val https_cb_id : channel_binding_id
+  (** Identifies a channel binding to anonymous HTTPS (i.e. no client
+      certificates), with or without web proxies.
+   *)
+
+val proxy_only_cb_id : channel_binding_id
+  (** Identifies a channel binding to web proxy connections. Use this to
+      e.g. send an FTP URL to a web proxy via HTTP
+   *)
+
+val new_cb_id : unit -> channel_binding_id
+  (** Allocates and returns a new ID *)
+
+val http_transport_channel_type : transport_channel_type
+  (** Transport via HTTP *)
+
+type proxy_type = [`Http_proxy | `Socks5 ] 
+
+
+(** {1 Pipelines} *)
+
 (** A pipeline is a queue of HTTP calls to perform *)
 class pipeline :
   object
@@ -869,12 +1119,12 @@ class pipeline :
 
     method set_proxy : string -> int -> unit
 	(** [set_proxy name port]:
-	 * sets that a proxy [name] listening on [port] is to be used
+	 * sets that an HTTP proxy [name] listening on [port] is to be used
 	 *)
 
     method set_proxy_auth : string -> string -> unit
-	(** sets user and password for the proxy. Only the "basic" 
-         * authentication method is implemented.
+	(** sets user and password for the proxy. Works for both "digest"
+	    and "basic" mechanisms. Any realm is acceptable.
 	 *)
 
     method avoid_proxy_for : string list -> unit
@@ -888,6 +1138,48 @@ class pipeline :
 	 * and set the proxy options from them.
 	 *)
 
+    method set_socks5_proxy : string -> int -> unit
+      (** Sets that a SOCKS version 5 proxy is used at this host and port.
+	  There is no authentication. The [avoid_proxy_for] setting is
+	  honoured.
+       *)
+
+    method configure_transport : 
+             channel_binding_id -> transport_channel_type -> unit
+       (** [configure_transport id transport]: Configures that messages with
+	   channel binding ID [id] are exchanged on [transport].
+
+	   By default, there is only a configuration for
+	   {!Http_client.http_cb_id}, i.e. for normal unencrypted channels.
+	*)
+
+    method set_transport_proxy : channel_binding_id ->
+                                 string ->
+                                 int ->
+                                 (string * string) option ->
+                                 proxy_type -> unit
+      (** [set_transport_proxy id host port auth ptype]: Sets a special
+	  proxy for the transport identified by [id]. This overrides
+	  [set_proxy], [set_proxy_auth], and [set_socks5_proxy] for the
+	  given transport.
+       *)
+
+    method set_transport_proxy_from_environment : 
+             (string * channel_binding_id) list -> unit
+      (** Like [set_proxy_from_environment], this method inspects environment
+	  variables and configures the proxy settings. This function, however,
+	  is more flexible, and can use different environment variables for
+	  different transports.
+
+	  The argument list has pairs [(var_name, id)] meaning that the
+	  environment variable [var_name] configures the proxy for [id].
+	  For instance, 
+	  {[ [("http_proxy", http_cb_id); ("https_proxy", https_cb_id)] ]}
+	  means that these two variables are used for the respective
+	  transports.
+
+	  The variable ["no_proxy"] is interpreted anyway.
+       *)
 
     method reset : unit -> unit
 	(** Empties the pipeline and inactivates any open connection.
@@ -922,6 +1214,24 @@ class pipeline :
 	 * indicated.
 	 *)
 
+    method add_e : http_call -> unit Uq_engines.engine
+      (** The same as engine: The call is added to the pipeline, and
+	  when it is processed, the returned engine transitions to the
+	  state [`Done()].
+       *)
+
+    method proxy_type : string -> proxy_type option
+      (** [proxy_type url] returns [Some pt] if a proxy would be used for this
+	  [url], and [None] if a direct connection would be made.
+       *)
+
+    method proxy_type_of_call : http_call -> proxy_type option
+      (** Same for an already created call object *)
+
+    method channel_binding : http_call -> channel_binding_id
+      (** Reports the current channel binding of this call *)
+
+
     method run : unit -> unit
       (** Runs through the requests in the pipeline. If a request can be
        * fulfilled, i.e. the server sends a response, the state of the
@@ -1026,6 +1336,21 @@ class pipeline :
   * ]}
  *)
 
+(** {2 Auxiliary pipeline functions} *)
+
+val parse_proxy_setting : string -> (string * int * (string * string) option)
+  (** Parses the value of an environment variable like [http_proxy],
+      i.e. an HTTP URL. The argument is the URL.
+      Returns [(host,port,auth)] where [auth] may include user name and
+      password.
+   *)
+
+val parse_no_proxy : string -> string list
+  (** Parses the value of an environment variable like [no_proxy]. Returns
+      the list of domains.
+   *)
+
+
 (** {1 Convenience module for simple applications} *)
 
 (** Do [open Http_client.Convenience] for simple applications. *)
@@ -1061,6 +1386,9 @@ sig
     * Requests are not repeated if there is a HTTP return code that indicates
     * a normal operating condition.
     * POST and DELETE requests are never repeated.
+    *
+    * Error codes are reported as {!Http_client.Http_error}. Note that
+    * this is different than what the pipeline core does.
     *)
 
   (** {b Thread safety}
@@ -1083,6 +1411,11 @@ sig
   val http_password : string ref
     (** The default password if authentication is required *)
 
+  val configure_pipeline : (pipeline -> unit) -> unit
+    (** This function will be called before the pipeline is used. This
+	is intended for fine-grained configuration.
+     *)
+
   val http_get_message : string -> http_call
     (** Does a "GET" request with the given URL and returns the message *)
 	  
@@ -1122,13 +1455,24 @@ sig
       * body.
      *)
 
-  val http_verbose : unit -> unit
-    (** Turns on debug messages on stderr. *)
+  val http_verbose : 
+        ?verbose_status:bool ->
+        ?verbose_request_header:bool ->
+        ?verbose_response_header:bool ->
+        ?verbose_request_contents:bool ->
+        ?verbose_response_contents:bool ->
+        ?verbose_connection:bool ->
+        ?verbose_events:bool ->
+        unit -> unit
+    (** Turns on debug messages on stderr. By default, all options are 
+	turned on.
+     *)
 
 end
 
-(**/**)
+(** {1 Debugging} *)
 
-val init_mt : 
-      create_lock_unlock_pair:(unit -> ( (unit->unit) * (unit->unit) ) ) ->
-      unit
+module Debug : sig
+  val enable : bool ref
+    (** Enables {!Netlog}-style debugging of this module *)
+end
