@@ -1,4 +1,4 @@
-(* $Id: netshm.ml 996 2006-09-19 15:39:34Z gerd $ *)
+(* $Id: netshm.ml 1589 2011-04-29 15:03:05Z gerd $ *)
 
 (* FILE FORMAT:
  *
@@ -239,8 +239,8 @@ let name_of_shm =
 
 let close_shm =
   function
-    | `File(_,fd,is_open) -> if !is_open then Unix.close fd
-    | `POSIX(_,fd,is_open) -> if !is_open then Unix.close fd
+    | `File(_,fd,is_open) -> if !is_open then Unix.close fd; is_open := false
+    | `POSIX(_,fd,is_open) -> if !is_open then Unix.close fd; is_open := false
 
 
 let unlink_shm =
