@@ -1,4 +1,4 @@
-(* $Id: netchannels.ml 1012 2006-10-02 13:12:56Z gerd $
+(* $Id: netchannels.ml 1613 2011-06-08 15:45:17Z gerd $
  * ----------------------------------------------------------------------
  *
  *)
@@ -7,6 +7,27 @@ exception Closed_channel
 exception Buffer_underrun
 exception Command_failure of Unix.process_status
 
+let () =
+  Netexn.register_printer
+    (Command_failure(Unix.WEXITED 0))
+    (fun e ->
+       match e with
+	 | Command_failure ps ->
+	     let ps_str =
+	       match ps with
+		 | Unix.WEXITED n -> "WEXITED " ^ string_of_int n
+		 | Unix.WSIGNALED n -> "WSIGNALED " ^ string_of_int n
+		 | Unix.WSTOPPED n -> "WSTOPPED " ^ string_of_int n
+	     in
+	     "Netchannels.Command_failure(" ^ ps_str ^ ")"
+	 | _ ->
+	     assert false
+    )
+
+let () =
+  Netsys_signal.init()
+
+
 class type rec_in_channel = object
   method input : string -> int -> int -> int
   method close_in : unit -> unit
@@ -171,7 +192,7 @@ end
 
 (****************************** input ******************************)
 
-class input_channel ch (* : in_obj_channel *) =
+class input_channel ?(onclose=fun () -> ()) ch (* : in_obj_channel *) =
 object (self)
   val ch = ch
   val mutable closed = false
@@ -205,8 +226,9 @@ object (self)
     Pervasives.input_byte ch 
 
   method close_in () =
-    if closed then self # complain_closed();
-    Pervasives.close_in ch; closed <- true
+    if not closed then (
+      Pervasives.close_in ch; closed <- true; onclose()
+    )
 
   method pos_in =
     if closed then self # complain_closed();
@@ -221,11 +243,12 @@ object (self)
   inherit input_channel ch as super
 
   method close_in() =
-    ignore(self # pos_in);   (* raise Closed_channel if necessary *)
-    let p = Unix.close_process_in ch in
-    closed <- true;
-    if p <> Unix.WEXITED 0 then
-      raise (Command_failure p);
+    if not closed then (
+      let p = Unix.close_process_in ch in
+      closed <- true;
+      if p <> Unix.WEXITED 0 then
+	raise (Command_failure p);
+    )
 end
 ;;
 
@@ -304,7 +327,6 @@ object (self)
 
 
   method close_in() =
-    if closed then self # complain_closed();
     str <- "";
     closed <- true;
 
@@ -396,7 +418,6 @@ object (self)
 
 
   method close_in() =
-    if closed then self # complain_closed();
     closed <- true;
 
 
@@ -467,6 +488,18 @@ let string_of_in_obj_channel (objch : in_obj_channel) : string =
 ;;
 
 
+let lines_of_in_obj_channel ch =
+  let acc = ref [] in
+  try
+    while true do
+      acc := ch#input_line() :: !acc
+    done;
+    assert false
+  with
+    | End_of_file -> List.rev !acc
+;;
+
+
 let with_in_obj_channel ch f =
   try
     let result = f ch in
@@ -556,9 +589,10 @@ object(self)
     n
 
   method close_in () =
-    if closed then raise Closed_channel;
-    closed <- true;
-    r # close_in()
+    if not closed then (
+      closed <- true;
+      r # close_in()
+    )
 
   method pos_in =
     if closed then raise Closed_channel;
@@ -624,8 +658,10 @@ object (self)
 
 
   method close_in () =
-    ch # close_in();
-    closed <- true
+    if not closed then (
+      ch # close_in();
+      closed <- true
+    )
 
   method pos_in =
     (ch # pos_in) - (buflen - bufpos)
@@ -834,6 +870,17 @@ let copy_channel
 
 class output_channel ?(onclose = fun () -> ()) ch 
 		     (* : out_obj_channel *) =
+  let errflag = ref false in
+  let monitored f arg =
+    try 
+      let r = f arg in
+      errflag := false;
+      r
+    with
+      | error ->
+	  errflag := true;
+	  raise error in
+
 object (self)
   val ch = ch
   val onclose = onclose
@@ -850,43 +897,72 @@ object (self)
     let p0 = Pervasives.pos_out ch in
     try 
       Pervasives.output ch buf pos len;
+      errflag := false;
       len
     with
-	Sys_blocked_io ->
+      | Sys_blocked_io ->
 	  let p1 = Pervasives.pos_out ch in
+	  errflag := false;
 	  p1 - p0
+      | error ->
+	  errflag := true;
+	  raise error
 
   method really_output buf pos len =
     if closed then self # complain_closed();
-    Pervasives.output ch buf pos len
+    monitored (Pervasives.output ch buf pos) len
 
   method output_char c =
     if closed then self # complain_closed();
-    Pervasives.output_char ch c
+    monitored (Pervasives.output_char ch) c
 
   method output_string s =
     if closed then self # complain_closed();
-    Pervasives.output_string ch s
+    monitored (Pervasives.output_string ch) s
 
   method output_byte b =
     if closed then self # complain_closed();
-    Pervasives.output_byte ch b
+    monitored (Pervasives.output_byte ch) b
 
   method output_buffer b =
     if closed then self # complain_closed();
-    Buffer.output_buffer ch b
+    monitored(Buffer.output_buffer ch) b
 
   method output_channel ?len ch =
     if closed then self # complain_closed();
-    ignore(copy_channel ?len ch (self : #out_obj_channel :> out_obj_channel))
+    ignore
+      (monitored
+	 (copy_channel ?len ch)
+	 (self : #out_obj_channel :> out_obj_channel))
 
   method flush() =
     if closed then self # complain_closed();
-    Pervasives.flush ch
+    monitored Pervasives.flush ch
 
   method close_out() =
-    if closed then self # complain_closed();
-    Pervasives.close_out ch; closed <- true; onclose()
+    if not closed then (
+      ( try
+	  (* if !errflag is set, we know that the immediately preceding
+	     operation raised an exception, and we are now likely in the
+	     exception handler
+	   *)
+	  if !errflag then
+	    Pervasives.close_out_noerr ch
+	  else
+	    Pervasives.close_out ch; 
+	  closed <- true; 
+	with
+	  | error ->
+	      let bt = Printexc.get_backtrace() in
+	      Netlog.logf `Err
+		"Netchannels.output_channel: \
+                   Suppressed error in close_out: %s - backtrace: %s"
+		(Netexn.to_string error) bt;
+	      Pervasives.close_out_noerr ch;
+	      closed <- true; 
+      );
+      onclose()
+    )
 
   method pos_out = 
     if closed then self # complain_closed();
@@ -897,18 +973,18 @@ end
 
 
 class output_command ?onclose cmd =
-let ch = Unix.open_process_out cmd in
+  let ch = Unix.open_process_out cmd in
 object (self)
   inherit output_channel ?onclose ch as super
 
   method close_out() =
-    ignore(self # pos_out);   (* raise Closed_channel if necessary *)
-    let p = Unix.close_process_out ch in
-    closed <- true;
-    onclose();
-    (* super # close_out(); *)
-    if p <> Unix.WEXITED 0 then
-      raise (Command_failure p);
+    if not closed then (
+      let p = Unix.close_process_out ch in
+      closed <- true;
+      onclose();
+      if p <> Unix.WEXITED 0 then
+	raise (Command_failure p);  (* Keep this *)
+    )
 end
 ;;
 
@@ -956,9 +1032,10 @@ object(self)
     ()
 
   method close_out() =
-    if closed then self # complain_closed();
-    closed <- true;
-    onclose()
+    if not closed then (
+      closed <- true;
+      onclose()
+    )
 
   method pos_out = 
     if closed then self # complain_closed();
@@ -1018,9 +1095,10 @@ object(self)
     ()
 
   method close_out() =
-    if closed then self # complain_closed();
-    closed <- true;
-    onclose()
+    if not closed then (
+      closed <- true;
+      onclose()
+    )
 
   method pos_out = 
     if closed then self # complain_closed();
@@ -1065,7 +1143,6 @@ object(self)
   method flush() = 
     if closed then self # complain_closed();
   method close_out() =
-    if closed then self # complain_closed();
     closed <- true
   method pos_out =
     if closed then self # complain_closed();
@@ -1076,6 +1153,10 @@ end ;;
 let with_out_obj_channel ch f =
   try
     let result = f ch in
+    (* we _have_ to flush here because close_out often does no longer
+       report exceptions
+     *)
+    ( try ch # flush() with Closed_channel -> ());
     ( try ch # close_out() with Closed_channel -> ());
     result
   with
@@ -1159,9 +1240,10 @@ object(self)
     r # flush();
 
   method close_out () =
-    if closed then raise Closed_channel;
-    closed <- true;
-    r # close_out()
+    if not closed then (
+      closed <- true;
+      r # close_out()
+    )
 
   method pos_out =
     if closed then raise Closed_channel;
@@ -1198,9 +1280,20 @@ object (self)
     ch # flush()
 
   method close_out() = 
-    self # flush();
-    ch # close_out();
-    closed <- true
+    if not closed then (
+      ( try
+	  self # flush()
+	with
+	  | error ->
+	      let bt = Printexc.get_backtrace() in
+	      Netlog.logf `Err
+		"Netchannels.buffered_raw_out_channel: \
+                  Suppressed error in close_out: %s - backtrace: %s"
+		(Netexn.to_string error) bt;
+      );
+      ch # close_out();
+      closed <- true
+    )
 
   method pos_out =
     (ch # pos_out) + bufpos
@@ -1229,6 +1322,7 @@ let lift_out ?(buffered=true) ?buffer_size (x : lift_out_arg) =
 (************************* raw channels *******************************)
 
 class input_descr_prelim ?(blocking=true) ?(start_pos_in = 0) fd =
+  let fd_style = Netsys.get_fd_style fd in
 object (self)
   val fd_in = fd
   val mutable pos_in = start_pos_in
@@ -1240,7 +1334,7 @@ object (self)
   method input buf pos len =
     if closed_in then self # complain_closed();
     try
-      let n = Unix.read fd_in buf pos len in
+      let n = Netsys.gread fd_style fd_in buf pos len in
       pos_in <- pos_in + n;
       if n=0 && len>0 then raise End_of_file;
       n
@@ -1250,15 +1344,18 @@ object (self)
       | Unix.Unix_error(Unix.EAGAIN,_,_)
       | Unix.Unix_error(Unix.EWOULDBLOCK,_,_) ->
 	  if blocking then (
-	    let _  = Netsys.restarting_select [fd] [] [] (-1.0) in
+	    let _  = Netsys.restart 
+	      (Netsys.wait_until_readable fd_style fd) (-1.0) in
 	    self # input buf pos len
 	  )
 	  else 0
 	  
   
   method close_in () =
-    if closed_in then self # complain_closed();
-    Unix.close fd_in; closed_in <- true
+    if not closed_in then (
+      Netsys.gclose fd_style fd_in; 
+      closed_in <- true
+    )
 
   method pos_in =
     if closed_in then self # complain_closed();
@@ -1273,6 +1370,7 @@ class input_descr ?blocking ?start_pos_in fd : raw_in_channel =
 
 
 class output_descr_prelim ?(blocking=true) ?(start_pos_out = 0) fd =
+  let fd_style = Netsys.get_fd_style fd in
 object (self)
   val fd_out = fd
   val mutable pos_out = start_pos_out
@@ -1284,7 +1382,7 @@ object (self)
   method output buf pos len =
     if closed_out then self # complain_closed();
     try
-      let n = Unix.single_write fd_out buf pos len in
+      let n = Netsys.gwrite fd_style fd_out buf pos len in
       pos_out <- pos_out + n;
       n
     with
@@ -1293,15 +1391,34 @@ object (self)
       | Unix.Unix_error(Unix.EAGAIN,_,_)
       | Unix.Unix_error(Unix.EWOULDBLOCK,_,_) ->
 	  if blocking then (
-	    let _  = Netsys.restarting_select [] [fd] [] (-1.0) in
+	    let _  = Netsys.restart 
+	      (Netsys.wait_until_writable fd_style fd) (-1.0) in
 	    self # output buf pos len
 	  )
 	  else
 	    0
   
   method close_out () =
-    if closed_out then self # complain_closed();
-    Unix.close fd_out; closed_out <- true
+    if not closed_out then (
+      ( try
+	  Netsys.gshutdown fd_style fd Unix.SHUTDOWN_SEND
+	with
+	  | Netsys.Shutdown_not_supported -> ()
+	  | Unix.Unix_error(Unix.EAGAIN, _, _) ->
+	      (* FIXME. We block here even when non-blocking semantics
+               is requested. We do this because most programmers would
+               be surprised to get EAGAIN when closing a channel.
+               Actually, this only affects Win32 output threads.
+	       *)
+	      let _  = Netsys.restart 
+		(Netsys.wait_until_writable fd_style fd) (-1.0) in
+	      Netsys.gshutdown fd_style fd Unix.SHUTDOWN_SEND
+	  | Unix.Unix_error(Unix.EPERM, _, _) ->
+	      ()
+      );
+      Netsys.gclose fd_style fd_out; 
+      closed_out <- true
+    )
 
   method pos_out =
     if closed_out then self # complain_closed();
@@ -1320,23 +1437,49 @@ class output_descr ?blocking ?start_pos_out fd : raw_out_channel =
 
 class socket_descr ?blocking ?(start_pos_in = 0) ?(start_pos_out = 0) fd 
       : raw_io_channel =
+  let fd_style = Netsys.get_fd_style fd in
+  let () =
+    match fd_style with
+      | `Recv_send _
+      | `Recv_send_implied
+      | `W32_pipe -> ()
+      | _ ->
+	  failwith "Netchannels.socket_descr: This type of descriptor is \
+                    unsupported"
+  in
 object (self)
   inherit input_descr_prelim ?blocking ~start_pos_in fd
   inherit output_descr_prelim ?blocking ~start_pos_out fd 
   
+  method private gen_close cmd =
+    ( try
+	Netsys.gshutdown fd_style fd cmd
+      with
+	| Netsys.Shutdown_not_supported -> ()
+	| Unix.Unix_error(Unix.EAGAIN, _, _) -> assert false
+	| Unix.Unix_error(Unix.EPERM, _, _) -> ()
+    );
+    if cmd = Unix.SHUTDOWN_ALL then
+      Netsys.gclose fd_style fd
+
+
   method close_in () =
-    if closed_in then self # complain_closed();
-    closed_in <- true;
-    Unix.shutdown fd Unix.SHUTDOWN_RECEIVE;
-    if closed_out then
-      Unix.close fd
-  
+    if not closed_in then (
+      closed_in <- true;
+      if closed_out then
+	self # gen_close Unix.SHUTDOWN_ALL
+      else
+	self # gen_close Unix.SHUTDOWN_RECEIVE
+    )
+
   method close_out () =
-    if closed_out then self # complain_closed();
-    closed_out <- true;
-    Unix.shutdown fd Unix.SHUTDOWN_SEND;
-    if closed_in then
-      Unix.close fd
+    if not closed_out then (
+      closed_out <- true;
+      if closed_in then
+	self # gen_close Unix.SHUTDOWN_ALL
+      else
+	self # gen_close Unix.SHUTDOWN_SEND
+    )
 end
 ;;
 
@@ -1346,46 +1489,63 @@ end
 class buffered_trans_channel ?(close_mode = (`Commit : close_mode)) 
 			      (ch : out_obj_channel)
                               : trans_out_obj_channel =
-  let _transbuf = Buffer.create 50 in
+  let closed = ref false in
+  let transbuf = ref(Buffer.create 50) in
+  let trans = ref(new output_buffer !transbuf) in
+  let reset() =
+    transbuf := Buffer.create 50;
+    trans := new output_buffer !transbuf in
 object (self)
-  val transbuf   = _transbuf
-  val trans      = new output_buffer _transbuf
   val out        = ch
   val close_mode = close_mode
 
-  method output         = trans # output
-  method really_output  = trans # really_output
-  method output_char    = trans # output_char
-  method output_string  = trans # output_string
-  method output_byte    = trans # output_byte
-  method output_buffer  = trans # output_buffer
-  method output_channel = trans # output_channel
-  method flush          = trans # flush
+  method output         = !trans # output
+  method really_output  = !trans # really_output
+  method output_char    = !trans # output_char
+  method output_string  = !trans # output_string
+  method output_byte    = !trans # output_byte
+  method output_buffer  = !trans # output_buffer
+  method output_channel = !trans # output_channel
+  method flush          = !trans # flush
 
   method close_out() =
-    ( match close_mode with
-	  `Commit   -> self # commit_work()
-	| `Rollback -> self # rollback_work()
-    );
-    trans # close_out();
-    out # close_out()
+    if not !closed then (
+      ( try
+	  ( match close_mode with
+		`Commit   -> self # commit_work()
+	      | `Rollback -> self # rollback_work()
+	  )
+	with
+	  | error ->
+	      let bt = Printexc.get_backtrace() in
+	      Netlog.logf `Err
+		"Netchannels.buffered_trans_channel: \
+                   Suppressed error in close_out: %s - backtrace: %s"
+		(Netexn.to_string error) bt;
+      );
+      !trans # close_out();
+      out # close_out();
+      closed := true
+    )
 
 
   method pos_out =
-    out # pos_out + trans # pos_out
+    out # pos_out + !trans # pos_out
 
   method commit_work() =
     try
-      out # output_buffer transbuf;
+      (* in any way avoid that the contents of transbuf are printed twice *)
+      let b = !transbuf in
+      reset();
+      out # output_buffer b;
       out # flush();
-      Buffer.reset transbuf
     with
 	err ->
 	  self # rollback_work();   (* reset anyway *)
 	  raise err
 
   method rollback_work() =
-    Buffer.reset transbuf
+    reset()
 
 end
 ;;
@@ -1394,7 +1554,7 @@ end
 let make_temporary_file 
   ?(mode = 0o600) 
   ?(limit = 1000) 
-  ?(tmp_directory = Filename.current_dir_name ) 
+  ?(tmp_directory = Netsys_tmp.tmp_directory() ) 
   ?(tmp_prefix = "netstring")
   () =
   (* Returns (filename, in_channel, out_channel). *)
@@ -1403,7 +1563,7 @@ let make_temporary_file
       let fn =
         Filename.concat
           tmp_directory
-          (tmp_prefix ^ "-" ^ (string_of_int n))
+          (Netsys_tmp.tmp_prefix tmp_prefix ^ "-" ^ (string_of_int n))
       in
       let fd_in =
 	Unix.openfile fn [ Unix.O_RDWR; Unix.O_CREAT; Unix.O_EXCL ] mode in
@@ -1445,12 +1605,14 @@ class tempfile_trans_channel ?(close_mode = (`Commit : close_mode))
 			      : trans_out_obj_channel =
   let _transname, _transch_in, _transch_out = 
     make_temporary_file ?tmp_directory ?tmp_prefix () in
+  let closed = ref false in
 object (self)
   val transch_out        = _transch_out
   val mutable transch_in = _transch_in
   val trans              = new output_channel _transch_out
   val mutable out        = ch
   val close_mode         = close_mode
+  val mutable need_clear = false
 
   initializer
     try
@@ -1470,29 +1632,44 @@ object (self)
 	  close_out _transch_out;
 	  raise err
 
-  method output         = trans # output
-  method really_output  = trans # really_output
-  method output_char    = trans # output_char
-  method output_string  = trans # output_string
-  method output_byte    = trans # output_byte
-  method output_buffer  = trans # output_buffer
-  method output_channel = trans # output_channel
-  method flush          = trans # flush
+  method output         = if need_clear then self#clear(); trans # output
+  method really_output  = if need_clear then self#clear(); trans # really_output
+  method output_char    = if need_clear then self#clear(); trans # output_char
+  method output_string  = if need_clear then self#clear(); trans # output_string
+  method output_byte    = if need_clear then self#clear(); trans # output_byte
+  method output_buffer  = if need_clear then self#clear(); trans # output_buffer
+  method output_channel = if need_clear then self#clear(); trans #output_channel
+  method flush          = if need_clear then self#clear(); trans # flush
 
   method close_out() =
-    ( match close_mode with
-	  `Commit   -> self # commit_work()
-	| `Rollback -> self # rollback_work()
-    );
-    Pervasives.close_in transch_in;
-    trans # close_out();      (* closes transch_out *)
-    out # close_out()
+    if not !closed then (
+      if need_clear then self#clear(); 
+      ( try
+	  ( match close_mode with
+		`Commit   -> self # commit_work()
+	      | `Rollback -> self # rollback_work()
+	  )
+	with
+	  | error ->
+	      let bt = Printexc.get_backtrace() in
+	      Netlog.logf `Err
+		"Netchannels.tempfile_trans_channel: \
+                  Suppressed error in close_out: %s - backtrace: %s"
+		(Netexn.to_string error) bt;
+      );
+      Pervasives.close_in transch_in;
+      trans # close_out();      (* closes transch_out *)
+      out # close_out();
+      closed := true
+    )
 
 
   method pos_out =
+    if need_clear then self#clear(); 
     out # pos_out + trans # pos_out
 
   method commit_work() =
+    need_clear <- true;
     let len = trans # pos_out in
     trans # flush();
     Pervasives.seek_in transch_in 0;
@@ -1531,6 +1708,7 @@ object (self)
     (* Note: the old transch_in will be automatically finalized, but the
      * underlying file descriptor will not be closed in this case
      *)
+    need_clear <- false
 
 end
 ;;
@@ -1561,9 +1739,9 @@ let call_input refill f arg =
 ;;
 
 
-class pipe ?(conv = id_conv) () : io_obj_channel =
-  let _incoming = Netbuffer.create 1 in
-  let _outgoing = Netbuffer.create 1 in
+class pipe ?(conv = id_conv) ?(buffer_size = 1024) () : io_obj_channel =
+  let _incoming = Netbuffer.create buffer_size in
+  let _outgoing = Netbuffer.create buffer_size in
 object(self)
   (* The properties as "incoming buffer" [output_super] are simply inherited
    * from [output_netbuffer]. The "outgoing buffer" [input_super] invocations
@@ -1645,31 +1823,38 @@ class output_filter
       (p : io_obj_channel) (out : out_obj_channel) : out_obj_channel =
 object(self)
   val p = p
+  val mutable p_closed = false  (* output side of p is closed *)
   val out = out
   val buf = String.create 1024  (* for copy_channel *)
 
   method output s pos len =
+    if p_closed then raise Closed_channel;
     let n = p # output s pos len in
     self # transfer();
     n
 
   method really_output s pos len =
+    if p_closed then raise Closed_channel;
     p # really_output s pos len;
     self # transfer();
 
   method output_char c =
+    if p_closed then raise Closed_channel;
     p # output_char c;
     self # transfer();
 
   method output_string s =
+    if p_closed then raise Closed_channel;
     p # output_string s;
     self # transfer();
 
   method output_byte b =
+    if p_closed then raise Closed_channel;
     p # output_byte b;
     self # transfer();
 
   method output_buffer b =
+    if p_closed then raise Closed_channel;
     p # output_buffer b;
     self # transfer();
 
@@ -1677,6 +1862,7 @@ object(self)
     (* To avoid large intermediate buffers, the channel is copied
      * chunk by chunk 
      *)
+    if p_closed then raise Closed_channel;
     let len_to_do = ref (match len with None -> -1 | Some l -> max 0 l) in
     let buf = buf in
     while !len_to_do <> 0 do
@@ -1696,8 +1882,20 @@ object(self)
     out # flush()
 
   method close_out() =
-    p # close_out();
-    self # transfer();
+    if not p_closed then (
+      p # close_out();
+      p_closed <- true;
+      ( try
+	  self # transfer()
+	with
+	  | error ->
+	      (* We report the error. However, we prevent that another,
+		 immediately following [close_out] reports the same
+		 error again. This is done by setting p_closed.
+	       *)
+	      raise error
+      )
+    )
 
   method pos_out = p # pos_out
 
