@@ -1,4 +1,4 @@
-(* $Id: netchannels.mli 1009 2006-10-01 20:26:01Z gerd $
+(* $Id: netchannels.mli 1613 2011-06-08 15:45:17Z gerd $
  * ----------------------------------------------------------------------
  *
  *)
@@ -59,6 +59,13 @@
  * process data, but is in non-blocking mode, both methods now return the
  * value 0. In previous releases of ocamlnet, the behaviour was not
  * defined.
+ *
+ * {b Ocamlnet-3.0} changed the behavior of [close_out]. Errors are no longer
+ * reported - instead, the exception is logged to {!Netlog}. For a stricter
+ * error handling, it is suggested to call [flush] first. Also, [close_in]
+ * and [close_out] no longer raise [Closed_channel] when the channel is
+ * already closed. Read more about this in the section
+ * {!Netchannels.rec_out_channel.close_error}.
  *)
 
 exception Closed_channel
@@ -110,9 +117,10 @@ class type rec_in_channel = object
   method close_in : unit -> unit
     (** Closes the channel for input.
      *
-     * When the channel is already closed, the exception [Closed_channel] will
-     * be raised if an ocamlnet implementation is used. For implementations
-     * of other libraries there is no standard for this case.
+     * When the channel is already closed, this is a no-op.
+     *
+     * Error policy: Exceptions are only raised in cases of serious
+     * corruption, e.g. if the underlying descriptor is invalid.
      *)
 end
 
@@ -160,15 +168,69 @@ class type rec_out_channel = object
      *)
   method flush : unit -> unit
     (** If there is a write buffer, it will be flushed. Otherwise, nothing
-     * happens
+     * happens.
      *)
   method close_out : unit -> unit
     (** Flushes the buffer, if any, and closes the channel for output.
      *
-     * When the channel is already closed, the exception [Closed_channel] will
-     * be raised if an ocamlnet implementation is used. For implementations
-     * of other libraries there is no standard for this case.
+     * When the channel is already closed, this is a no-op.
      *)
+
+ (** {2:close_error How to close channels in case of errors}
+
+     The [close_out] method has actually two tasks: First, it writes out
+     all remaining data (like [flush]), and second, it releases OS
+     resources (e.g. closes file descriptors). There is the question
+     what has to happen when the write part fails - is the resource released
+     anyway or not?
+
+     We choose here a pragmatic approach under the assumption that
+     an OS error at close time is usually unrecoverable, and it is
+     more important to release the OS resource. Also, we
+     assume that the user is wise enough to call [flush] first if
+     it is essential to know write errors at close time. Under these
+     assumptions:
+
+     - The [flush] method fully reports any errors when writing out
+       the remaining data.
+     - When [flush] raises an error exception, it should discard
+       any data in the buffer. This is not obligatory, however,
+       but considered good practice, and is subject to discussion.
+     - The [close_out] method usually does not report errors by
+       raising exceptions, but only by logging them via {!Netlog}.
+       The OS resource is released in any case. As before, this
+       behavior is not obligatory, but considered as good practice,
+       and subject to discussion.
+
+     This ensures that the following code snippet reports all errors, but also
+     releases OS resources:
+     
+     {[
+       try 
+         ch # flush();
+         ch # close_out();
+       with error -> 
+          ch # close_out(); raise error
+     ]}
+
+     There are some cases where data can be first written when it is
+     known that the channel is closed. These data would not be written
+     by a preceding [flush]. In such cases:
+
+     - The best way to deal with it is to define another method,
+       e.g. called [write_eof], that marks the data as logically 
+       being complete, so a following [flush] can do the complete
+       shutdown cycle of the channel.
+     - At least, however, one should allow then that a double
+       [close_out] releases the descriptor: the first [close_out]
+       will report the error condition as exception, but discard
+       all data in the channel. The second [close_out] finally
+       releases the OS resource.
+
+     In any way, hard errors indicating bugs of the program logic
+     (like invalid file descriptors) should always be immediately
+     reported.
+  *)
 end
 
 (** Basic Unix-level class type for output channels as used by ocamlnet. In addition
@@ -309,6 +371,7 @@ end
 (** {1:input Input channels} *)
 
 class input_channel :
+  ?onclose:(unit -> unit) ->
   in_channel ->
     in_obj_channel
   (** Creates an input channel from an [in_channel], which must be open.
@@ -318,6 +381,8 @@ class input_channel :
    * non-seekable channels.
    *
    * The method [close_in] also closes the underlying [in_channel].
+   *
+   * The function [onclose] is called after the [in_channel] has been closed.
    *)
 
 
@@ -383,6 +448,13 @@ val string_of_in_obj_channel : in_obj_channel -> string
    * This function does not work for non-blocking channels.
    *)
 
+val lines_of_in_obj_channel : in_obj_channel -> string list
+  (** Reads from the input channel until EOF and returns the lines
+   * as string list. The input channel is not closed.
+   *
+   * This function does not work for non-blocking channels.
+   *)
+
 val with_in_obj_channel : 
   (#in_obj_channel as 'a) -> ('a -> 'b) -> 'b
   (** [with_in_obj_channel ch f]:
@@ -406,6 +478,9 @@ class output_channel :
    * non-seekable channels.
    *
    * The method [close_out] also closes the underlying [out_channel].
+   * There is some implicit logic to either use [close_out] or [close_out_noerr]
+   * depending on whether the immediately preceding operation already reported
+   * an error.
    *
    * @param onclose this function is called when the [close_out] method is
    * invoked, just after the underlying [out_channel] has been closed.
@@ -423,7 +498,8 @@ class output_command :
    *
    * When [close_out] is invoked, the subprocess is [wait]ed for. If the
    * process exits with code 0, the method returns normally. Otherwise,
-   * the exception [Command_failure] is raised.
+   * the exception [Command_failure] is raised. (The channel is closed
+   * even if this exception is raised.)
    *
    * @param onclose this function is called when the [close_out] method is
    * invoked, just after the underlying descriptor has been closed.
@@ -720,6 +796,9 @@ class input_descr :
    *
    * The method [close_in] also closes the file descriptor.
    *
+   * This class also supports Win32 proxy descriptors referring to an input
+   * channel.
+   *
    * @param blocking Whether the channel waits for data if it is not
    * possible to read from the (non-blocking) descriptor. Defaults to [true].
    * @param start_pos_in The position to which [pos_in] is initialized when
@@ -740,6 +819,9 @@ class output_descr :
    *
    * The method [close_out] also closes the file descriptor.
    *
+   * This class also supports Win32 proxy descriptors referring to an output
+   * channel.
+   *
    * @param blocking Whether the channel waits until it can output if it is not
    * possible to write to the (non-blocking) descriptor. Defaults to [true].
    * @param start_pos_out The position to which [pos_out] is initialized when
@@ -761,6 +843,10 @@ class socket_descr :
    *
    * The [pos_in] and [pos_out] methods returns logical positions.
    *
+   * This class supports sockets and Win32 named pipes. Note, however,
+   * that for Win32 named pipes it is not possible to shut down only one
+   * direction of the bidirectional data channel.
+   *
    * @param blocking See {!input_descr} and {!output_descr}
    * @param start_pos_in The position to which [pos_in] is initialized when
    * the channel is created, by default 0
@@ -796,9 +882,9 @@ val make_temporary_file :
    * the triple (name, inch, outch) containing the file [name],
    * the file opened as in_channel [inch] and as out_channel [outch].
    *
-   * @param tmp_directory By default the current directory
-   * @param tmp_prefix By default ["netstring"]. It is better to have a prefix
-   *   that is likely to be unique, e.g. the process ID, or the current time.
+   * @param tmp_directory Defaults to {!Netsys_tmp.tmp_directory()}
+   * @param tmp_prefix By default ["netstring"]. This needs not to be
+   *   unique, but just descriptive.
    * @param mode The creation mask of the file; defaults to 0o600, i.e. the
    *   file is private for the current user
    * @param limit Limits the number of trials to find the unique suffix.
@@ -833,6 +919,7 @@ class tempfile_trans_channel :
 
 class pipe :
   ?conv:(Netbuffer.t -> bool -> Netbuffer.t -> unit) ->
+  ?buffer_size:int -> 
   unit ->
     io_obj_channel
   (** A [pipe] has two internal buffers (realized by Netbuffer). The
