@@ -1,4 +1,4 @@
-(* $Id: netconversion.ml 1084 2007-02-20 12:36:17Z gerd $
+(* $Id: netconversion.ml 1588 2011-04-28 13:59:54Z gerd $
  * ----------------------------------------------------------------------
  *)
 
@@ -224,9 +224,9 @@ let rec is_single_byte =
 ;;
 
 
-let punct_re = Netstring_pcre.regexp "[\\-\\_\\.]";;
-let ibm_re = Netstring_pcre.regexp "^IBM(\\d+)$";;
-let year_re = Netstring_pcre.regexp ":\\d\\d\\d\\d$";;
+let punct_re = Netstring_str.regexp "[-_.]";;
+let ibm_re = Netstring_str.regexp "^IBM\\([0-9]+\\)$";;
+let year_re = Netstring_str.regexp ":[0-9][0-9][0-9][0-9]$";;
 
 
 let norm_enc_name e =
@@ -234,11 +234,11 @@ let norm_enc_name e =
    * converts "IBM#" to "CP#"; drops ":YEAR" suffixes 
    *)
   let e1 = String.uppercase e in
-  let e2 = Netstring_pcre.global_replace punct_re "" e1 in
-  let e3 = Netstring_pcre.global_replace year_re "" e2 in
-  match Netstring_pcre.string_match ibm_re e3 0 with
+  let e2 = Netstring_str.global_replace punct_re "" e1 in
+  let e3 = Netstring_str.global_replace year_re "" e2 in
+  match Netstring_str.string_match ibm_re e3 0 with
       Some r ->
-	"CP" ^ Netstring_pcre.matched_group r 1 e3
+	"CP" ^ Netstring_str.matched_group r 1 e3
     | None ->
 	e3
 ;;
@@ -251,6 +251,9 @@ let names =
    * `Enc_subset is missing in this list, of course.
    *
    * http://www.iana.org/assignments/character-sets
+   *
+   * A good reference is also:
+   * http://www.firstobject.com/character-set-name-alias-code-page.htm
    *)
   [ `Enc_utf16,        [ "UTF-16"; "UTF16"; "UCS2"; "ISO10646UCS2" ];
     `Enc_utf16_be,     [ "UTF-16BE"; "UTF16BE" ];
@@ -258,7 +261,7 @@ let names =
     `Enc_utf8,         [ "UTF-8"; "UTF8" ];
     `Enc_java,         [ "UTF-8-JAVA"; "UTF8JAVA"; "JAVA" ];
     `Enc_usascii,      [ "US-ASCII"; "USASCII"; "ASCII"; "ISO646US"; "CP367"; 
-			 "ISOIR6" ];
+			 "ISOIR6"; "ANSIX341968" ];
     `Enc_iso88591,     [ "ISO-8859-1"; "ISO88591"; "LATIN1"; "CP819"; 
 			 "ISOIR100" ];
     `Enc_iso88592,     [ "ISO-8859-2"; "ISO88592"; "LATIN2"; "ISOIR101";
@@ -501,7 +504,6 @@ let rec required_charsets (e : encoding) =
 ;;
 
 
-
 let rec same_encoding e1 e2 =
   match (e1,e2) with
       (`Enc_subset(e1_sub, f1), `Enc_subset(e2_sub, f2)) ->
@@ -554,6 +556,85 @@ let available_output_encodings() =
 ;;
 
 
+let (win32_code_pages : (_ * encoding) list) =
+  [  65001, `Enc_utf8;
+     1200,  `Enc_utf16_le;
+     1201,  `Enc_utf16_be;
+     20127, `Enc_usascii;
+     28591, `Enc_iso88591;
+     28592, `Enc_iso88592;
+     28593, `Enc_iso88593;
+     28594, `Enc_iso88594;
+     28595, `Enc_iso88595;
+     28596, `Enc_iso88596;
+     28597, `Enc_iso88597;
+     28598, `Enc_iso88598;
+     28599, `Enc_iso88599;
+     (* `Enc_iso885910 *)
+     (* `Enc_iso885911 *)
+     28603, `Enc_iso885913;
+     (* `Enc_iso885914 *)
+     28605, `Enc_iso885915;
+     (* `Enc_iso885916 *)
+     20866, `Enc_koi8r;
+     (* `Enc_jis0201 *)
+     20932, `Enc_eucjp;
+     51949, `Enc_euckr;
+     1250, `Enc_windows1250;
+     1251, `Enc_windows1251;
+     1252, `Enc_windows1252;
+     1253, `Enc_windows1253;
+     1254, `Enc_windows1254;
+     1255, `Enc_windows1255;
+     1256, `Enc_windows1256;
+     1257, `Enc_windows1257;
+     1258, `Enc_windows1258;
+     437, `Enc_cp437;
+     737, `Enc_cp737;
+     775, `Enc_cp775;
+     850, `Enc_cp850;
+     852, `Enc_cp852;
+     855, `Enc_cp855;
+     (* `Enc_cp856 *)
+     857, `Enc_cp857;
+     860, `Enc_cp860;
+     861, `Enc_cp861;
+     862, `Enc_cp862;
+     863, `Enc_cp863;
+     864, `Enc_cp864;
+     865, `Enc_cp865;
+     866, `Enc_cp866;
+     869, `Enc_cp869;
+     874, `Enc_cp874;
+     (* `Enc_cp1006 *)
+     37, `Enc_cp037;
+     20424, `Enc_cp424;
+     500, `Enc_cp500;
+     875, `Enc_cp875;
+     1026, `Enc_cp1026;
+     1047, `Enc_cp1047;
+     (* `Enc_adobe_standard_encoding *)
+     (* `Enc_adobe_symbol_encoding *)
+     (* `Enc_adobe_zapf_dingbats_encoding *)
+     10000, `Enc_macroman;
+  ]
+
+let user_encoding() =
+  match Sys.os_type with
+    | "Win32" ->
+	let cp = Netsys_win32.get_active_code_page() in
+	( try Some(List.assoc cp win32_code_pages)
+	  with Not_found -> None
+	)
+    | _ ->
+	( try
+	    let codeset = 
+	      (Netsys_posix.query_langinfo "").Netsys_posix.nl_CODESET in
+	    Some(encoding_of_string codeset)
+	  with
+	    | _ -> None
+	)
+
 (* Internal conversion interface:
  *
  * let (n_char, n_byte, enc') = read_XXX slice_char slice_blen s_in p_in l_in:
