@@ -1,4 +1,4 @@
-(* $Id: netdate.mli 1003 2006-09-24 15:17:15Z gerd $
+(* $Id: netdate.mli 1393 2010-01-17 21:24:44Z gerd $
  * ----------------------------------------------------------------------
  *
  *)
@@ -25,13 +25,55 @@ val localzone : int
 val create : ?zone:int -> float -> t
   (** Convert the time (seconds since the epoch) to a date/time record *)
 
-val parse : string -> t
-  (** Parse a string and return a date/time record *)
+val parse : ?zone:int -> string -> t
+  (** Parse a string and return a date/time record.
+
+      The following particles are recognized (by example):
+      - Date: [1971/06/22]
+      - Date: [06/22/1971]
+      - Date: [1971-06-22]
+      - Date: [22-June-1971]
+      - Date: [22.06.1971]
+      - Date: [June 22, 1971]
+      - Date: [22 June 1971]
+      - Date (2 digit year): [06/22/71]
+      - Date (2 digit year): [22.06.71]
+      - Date (2 digit year): [71-06-22]
+      - Date (2 digit year): [22-June-71]
+      - Month names ([June], [Jun])
+      - Weekday names ([Monday], [Mon])
+      - Time: [14:55]
+      - Time: [14:55:28]
+      - Time: [14:55:28.6754]   (the fractional part is not returned)
+      - Time may be preceded by [T]
+      - Time zone: identifiers like [UTC], [CET], or [Z]
+      - Time zone: [+01:00], [-01:00], only following time
+      - Time zone: [+0100], [-0100], only following time
+
+      Years must have 2 or 4 digits. 2-digit years >= 70 are interpreted
+      as [1900+x]. 2-digit years < 70 are interpreted as [2000+x].
+      Support for 2-digit years will be removed in a future version
+      of Ocamlnet. (Support for 3-digit years is already removed in
+      Ocamlnet 3.0.)
+
+      Only English names of months and weekdays are recognized.
+
+      A date must be given. Time, time zones, and weekdays are optional.
+      A missing time is reported as "00:00:00". A missing weekday is
+      reported by setting [week_day=(-1)]. A missing time zone is 
+      reported by setting [zone] to the passed default.
+
+      It is not checked whether the parsed numbers make sense
+      (e.g. whether months are between 1 and 12).
+
+      Date/time strings as defined in RFC 3339 are supported since
+      Ocamlnet 3.0.
+   *)
 
 val since_epoch : t -> float
   (** Convert a date/time record into the time (seconds since the epoch) *)
 
-val parse_epoch : string -> float
+val parse_epoch : ?zone:int -> string -> float
   (** Parse a string and return the time (seconds since the epoch *)
 
 val format_to : Netchannels.out_obj_channel -> fmt:string -> t -> unit
@@ -87,6 +129,7 @@ val format_to : Netchannels.out_obj_channel -> fmt:string -> t -> unit
    *        stands for east of UTC, a minus sign for west of UTC, hours and
    *        minutes follow with two digits each and no delimiter between them
    *        (common form for RFC 822 date headers).
+   *  - [%:z]: time zone with colon, e.g. +05:00 (new since Ocamlnet 3)
    *  - [%%]: a `%' character.
    *
    *)
@@ -109,3 +152,11 @@ val mk_usenet_date : ?zone:int -> float -> string
    *
    * Note that this format has only two digits for the year.
    *)
+
+val mk_internet_date : ?zone:int -> float -> string
+  (** Convert the time (seconds since the epoch) to a date string that
+   * conforms to RFC 3339. This is the most modern format, and should
+   * be used if permitted by the network protocol.
+   *
+   * Example: ["1996-12-19T16:39:57.89-08:00"].
+   *)
