@@ -1,4 +1,4 @@
-(* $Id: generate.ml 1062 2006-12-17 20:17:36Z gerd $
+(* $Id: generate.ml 1444 2010-04-25 23:14:52Z gerd $
  * ----------------------------------------------------------------------
  *
  *)
@@ -272,6 +272,9 @@ let output_type_declarations (f:formatter) (dl:xdr_def list) =
       | T_string _
       | T_string_unlimited ->
 	  fprintf f "string"
+      | T_mstring(_,_)
+      | T_mstring_unlimited _ ->
+	  fprintf f "Xdr_mstring.mstring"
       | T_option t' ->
           fprintf f "@[<hv 2>";
 	  output_type t';
@@ -550,6 +553,14 @@ let output_xdr_type (mli:formatter) (f:formatter) (dl:xdr_def list) =
 	  fprintf f "@]";
       | T_string_unlimited ->
 	  fprintf f "Xdr.x_string_max"
+      | T_mstring(name,n) ->
+	  fprintf f "@[<hv 2>Xdr.X_mstring(@,";
+	  fprintf f "%S" name;
+	  fprintf f ",@ ";
+	  output_uint4 f (constant !n);
+	  fprintf f ")@]";
+      | T_mstring_unlimited name ->
+	  fprintf f "(Xdr.x_mstring_max %S)" name
       | T_option t' ->
 	  fprintf f "@[<hv 2>Xdr.x_optional@ (";
 	  output_type rectypes t';
@@ -819,6 +830,9 @@ let output_conversions (mli:formatter) (f:formatter) (dl:xdr_def list) =
 	| T_string _
 	| T_string_unlimited ->
 	    fprintf f "(Xdr.dest_xv_string %s)" var
+	| T_mstring(_,_)
+	| T_mstring_unlimited _ ->
+	    fprintf f "(Xdr.dest_xv_mstring %s)" var
 	| T_option t' ->
 	    fprintf f "@[<hv>";
 	    fprintf f "(match Xdr.dest_xv_union_over_enum_fast %s with@ " var;
@@ -1108,11 +1122,27 @@ let output_conversions (mli:formatter) (f:formatter) (dl:xdr_def list) =
   )
 
   and output_toconv_for_array var t' =
-    fprintf f "@[<hv>";
-    fprintf f "(Array.map@;<1 2>(fun x -> ";
-    output_toconv_for_type "x" t';
-    fprintf f ")@ (Xdr.dest_xv_array %s))" var;
-    fprintf f "@]";
+    let t1 = get_type_from_map typemap t' in
+    match t1 with
+      | T_string _
+      | T_string_unlimited ->
+	  fprintf f "@[<hv>";
+	  fprintf f "(match %s with@;" var;
+	  fprintf f "| Xdr.XV_array x ->@;<1 4>";
+	  fprintf f "Array.map@;<1 6>(fun x -> ";
+	  output_toconv_for_type "x" t';
+	  fprintf f ")@;<1 6>x@ ";
+	  fprintf f "| Xdr.XV_array_of_string_fast x ->@;<1 4>";
+	  fprintf f "x@ ";
+	  fprintf f "| _ -> raise Xdr.Dest_failure@ ";
+	  fprintf f ")";
+	  fprintf f "@]"
+      | _ ->
+	  fprintf f "@[<hv>";
+	  fprintf f "(Array.map@;<1 2>(fun x -> ";
+	  output_toconv_for_type "x" t';
+	  fprintf f ")@ (Xdr.dest_xv_array %s))" var;
+	  fprintf f "@]";
 
   and output_toconv_for_tuple var tl =
     fprintf f "@[<hv>";
@@ -1187,6 +1217,9 @@ let output_conversions (mli:formatter) (f:formatter) (dl:xdr_def list) =
 	| T_string _
 	| T_string_unlimited ->
 	    fprintf f "(Xdr.XV_string %s)" var
+	| T_mstring(_,_)
+	| T_mstring_unlimited _ ->
+	    fprintf f "(Xdr.XV_mstring %s)" var
 	| T_option t' ->
 	    fprintf f "@[<hv>";
 	    fprintf f "(match %s with@ " var;
@@ -1428,12 +1461,18 @@ let output_conversions (mli:formatter) (f:formatter) (dl:xdr_def list) =
     fprintf f "@]"
   )
   and output_ofconv_for_array name var t' =
-    fprintf f "@[<hv 2>Xdr.XV_array@ ";
-    fprintf f "@[<hv 2>(Array.map@ ";
-    fprintf f "(fun x -> ";
-    output_ofconv_for_type name "x" t';
-    fprintf f ")@ %s)@]@]" var;
-
+    let t1 = get_type_from_map typemap t' in
+    match t1 with
+      | T_string _
+      | T_string_unlimited ->
+	  fprintf f "@[<hv 2>(Xdr.XV_array_of_string_fast %s)@]" var
+      | _ ->
+	  fprintf f "@[<hv 2>Xdr.XV_array@ ";
+	  fprintf f "@[<hv 2>(Array.map@ ";
+	  fprintf f "(fun x -> ";
+	  output_ofconv_for_type name "x" t';
+	  fprintf f ")@ %s)@]@]" var
+	  
   and output_ofconv_for_tuple name var tl =
     fprintf f "@[<hv 1>";
     fprintf f "(let (";
@@ -1614,14 +1653,18 @@ let output_progdefs (mli:formatter) (f:formatter) (dl:xdr_def list) =
 let output_client (mli:formatter) (f:formatter) (dl:xdr_def list) auxname =
 
   let rec check_program prog =
+    (* Make functor: *)
+    
     (* MLI: *)
     fprintf mli "@[<v>";
-    fprintf mli "@[<v 2>module %s : sig@ " prog.prog_symbol.ocaml_name;
+    fprintf mli "@[<v 2>module Make'%s(U'C:Rpc_client.USE_CLIENT) : sig@ "
+      prog.prog_symbol.ocaml_name;
     (* ML: *)
     fprintf f "@[<v>";
-    fprintf f "@[<v 2>module %s = struct@ " prog.prog_symbol.ocaml_name;
+    fprintf f "@[<v 2>module Make'%s(U'C:Rpc_client.USE_CLIENT) = struct@ " 
+      prog.prog_symbol.ocaml_name;
     (* Both: *)
-    List.iter (check_version prog) prog.prog_def;
+    List.iter (check_version `Make prog) prog.prog_def;
     (* MLI: *)
     fprintf mli "@]@ ";
     fprintf mli "end@ ";
@@ -1631,7 +1674,29 @@ let output_client (mli:formatter) (f:formatter) (dl:xdr_def list) auxname =
     fprintf f "end@ ";
     fprintf f "@]@\n";
 
-  and check_version prog vers =
+    (* Mapping with U'C=Rpc_client: *)
+
+    (* MLI: *)
+    fprintf mli "@[<v>";
+    fprintf mli "@[<v 2>module %s : sig@ "
+      prog.prog_symbol.ocaml_name;
+    (* ML: *)
+    fprintf f "@[<v>";
+    fprintf f "@[<v 2>module %s = struct@ " 
+      prog.prog_symbol.ocaml_name;
+    (* Both: *)
+    List.iter (check_version `Client prog) prog.prog_def;
+    (* MLI: *)
+    fprintf mli "@]@ ";
+    fprintf mli "end@ ";
+    fprintf mli "@]@\n";
+    (* ML: *)
+    fprintf f "@]@ ";
+    fprintf f "end@ ";
+    fprintf f "@]@\n";
+    
+
+  and check_version inst prog vers =
     let pv =
       prog.prog_symbol.ocaml_name ^ "'" ^ vers.version_symbol.ocaml_name in
     (* MLI: *)
@@ -1639,46 +1704,66 @@ let output_client (mli:formatter) (f:formatter) (dl:xdr_def list) auxname =
     fprintf mli "@[<v 2>module %s : sig" vers.version_symbol.ocaml_name;
     fprintf mli "@ ";
     fprintf mli "open %s@ " auxname;
-    fprintf mli "val @[<hv 4>create_client :@ ?esys:Unixqueue.event_system ->@ ?program_number:Rtypes.uint4 -> @ ?version_number:Rtypes.uint4 -> @ Rpc_client.connector ->@ Rpc.protocol ->@ Rpc_client.t@]";
-    fprintf mli "@ ";
-    fprintf mli "val @[<hv 4>create_portmapped_client :@ ?esys:Unixqueue.event_system ->@ ?program_number:Rtypes.uint4 -> @ ?version_number:Rtypes.uint4 -> @ string ->@ Rpc.protocol ->@ Rpc_client.t@]";
-    fprintf mli "@ ";
-    fprintf mli "val @[<hv 4>create_client2 :@ ?esys:Unixqueue.event_system ->@ ?program_number:Rtypes.uint4 -> @ ?version_number:Rtypes.uint4 -> @ Rpc_client.mode2 ->@ Rpc_client.t@]";
-    fprintf mli "@ ";
+    ( match inst with
+	| `Client ->
+	    fprintf mli "type t = Rpc_client.t@ ";
+	    fprintf mli "val @[<hv 4>create_client :@ ?esys:Unixqueue.event_system ->@ ?program_number:Rtypes.uint4 -> @ ?version_number:Rtypes.uint4 -> @ Rpc_client.connector ->@ Rpc.protocol ->@ Rpc_client.t@]";
+	    fprintf mli "@ ";
+	    fprintf mli "val @[<hv 4>create_portmapped_client :@ ?esys:Unixqueue.event_system ->@ ?program_number:Rtypes.uint4 -> @ ?version_number:Rtypes.uint4 -> @ string ->@ Rpc.protocol ->@ Rpc_client.t@]";
+	    fprintf mli "@ ";
+	    fprintf mli "val @[<hv 4>create_client2 :@ ?esys:Unixqueue.event_system ->@ ?program_number:Rtypes.uint4 -> @ ?version_number:Rtypes.uint4 -> @ Rpc_client.mode2 ->@ Rpc_client.t@]";
+	    fprintf mli "@ ";
+	| `Make ->
+	    fprintf mli "type t = U'C.t@ ";
+    );
+    fprintf mli "val _program : Rpc_program.t@ ";
     (* ML: *)
     fprintf f "@[<v>";
     fprintf f "@[<v 2>module %s = struct@ " vers.version_symbol.ocaml_name;
-    fprintf f "open %s@ " auxname;
-    fprintf f "let _program = program_%s@ " pv;
+    ( match inst with
+	| `Client ->
+	    (* Ocaml doesn't like: Make'<prog>(Rpc_client).<vers> *)
+	    fprintf f "module M'0 = Make'%s(Rpc_client)@ "
+	      prog.prog_symbol.ocaml_name ;
+	    fprintf f "include M'0.%s@ " 
+	      vers.version_symbol.ocaml_name;
+	    fprintf f "open %s@ " auxname;
+	    fprintf f "let _program = program_%s@ " pv;
 
-    fprintf f "@ ";
-    fprintf f "@[<hv 2>let create_client@ ";
-    fprintf f "?(esys = Unixqueue.create_unix_event_system())@ ";
-    fprintf f "?program_number@ ";
-    fprintf f "?version_number@ ";
-    fprintf f "connector@ ";
-    fprintf f "protocol =@ ";
-    fprintf f "  Rpc_client.create ?program_number ?version_number esys connector protocol _program";
-    fprintf f "@]";
+	    fprintf f "@ ";
+	    fprintf f "@[<hv 2>let create_client@ ";
+	    fprintf f "?(esys = Unixqueue.create_unix_event_system())@ ";
+	    fprintf f "?program_number@ ";
+	    fprintf f "?version_number@ ";
+	    fprintf f "connector@ ";
+	    fprintf f "protocol =@ ";
+	    fprintf f "  Rpc_client.create ?program_number ?version_number esys connector protocol _program";
+	    fprintf f "@]";
 
-    fprintf f "@ @ ";
-    fprintf f "@[<hv 2>let create_portmapped_client ?esys ?program_number ?version_number host protocol =@ ";
-    fprintf f "create_client ?esys ?program_number ?version_number (Rpc_client.Portmapped host) protocol";
-    fprintf f "@]";
+	    fprintf f "@ @ ";
+	    fprintf f "@[<hv 2>let create_portmapped_client ?esys ?program_number ?version_number host protocol =@ ";
+	    fprintf f "create_client ?esys ?program_number ?version_number (Rpc_client.Portmapped host) protocol";
+	    fprintf f "@]";
+	    
+	    fprintf f "@ @ ";
+	    fprintf f "@[<hv 2>let create_client2@ ";
+	    fprintf f "?(esys = Unixqueue.create_unix_event_system())@ ";
+	    fprintf f "?program_number@ ";
+	    fprintf f "?version_number@ ";
+	    fprintf f "mode2 =@ ";
+	    fprintf f "  Rpc_client.create2 ?program_number ?version_number mode2 _program esys";
+	    fprintf f "@]";
 
-    fprintf f "@ @ ";
-    fprintf f "@[<hv 2>let create_client2@ ";
-    fprintf f "?(esys = Unixqueue.create_unix_event_system())@ ";
-    fprintf f "?program_number@ ";
-    fprintf f "?version_number@ ";
-    fprintf f "mode2 =@ ";
-    fprintf f "  Rpc_client.create2 ?program_number ?version_number mode2 _program esys";
-    fprintf f "@]";
-
-    fprintf f "@ @ ";
+	    fprintf f "@ ";
+	| `Make ->
+	    fprintf f "open %s@ " auxname;
+	    fprintf f "let _program = program_%s@ " pv;
+	    fprintf f "type t = U'C.t@ ";
+	    fprintf f "@ ";
+    );
 
     (* Both: *)
-    List.iter (define_procedure prog vers) vers.version_def;
+    List.iter (define_procedure inst prog vers) vers.version_def;
 
     (* MLI: *)
     fprintf mli "@]@ end@ @]";
@@ -1687,35 +1772,46 @@ let output_client (mli:formatter) (f:formatter) (dl:xdr_def list) auxname =
     fprintf f "end@ ";
     fprintf f "@]";
 
-  and define_procedure prog vers proc =
+  and define_procedure inst prog vers proc =
     let pvp = prog.prog_symbol.ocaml_name ^ "'" ^
 	      vers.version_symbol.ocaml_name ^ "'" ^
 	      proc.proc_symbol.ocaml_name in
+    let cm =
+      match inst with
+	| `Client -> "Rpc_client"
+	| `Make -> "U'C" in
+
     (* MLI: *)
-    fprintf mli "val @[<hv 4>%s :@ Rpc_client.t ->@ %s ->@ %s@]@ "
+    fprintf mli "val @[<hv 4>%s :@ %s.t ->@ %s ->@ %s@]@ "
       proc.proc_symbol.ocaml_name
+      cm
       ("t_" ^ pvp ^ "'arg")
       ("t_" ^ pvp ^ "'res");
-    fprintf mli "val @[<hv 4>%s'async :@ Rpc_client.t ->@ %s ->@ ((unit -> %s) -> unit) ->@ unit@]@ "
+    fprintf mli "val @[<hv 4>%s'async :@ %s.t ->@ %s ->@ ((unit -> %s) -> unit) ->@ unit@]@ "
       proc.proc_symbol.ocaml_name
+      cm
       ("t_" ^ pvp ^ "'arg")
       ("t_" ^ pvp ^ "'res");
     (* ML: *)
-    fprintf f "@[<hv 2>";
-    fprintf f "let %s client arg =@ " proc.proc_symbol.ocaml_name;
-    (* fprintf f "assert(Rpc_client.program client == _program);@ "; *)
-    fprintf f "_to_%s'res (Rpc_client.sync_call client \"%s\" (_of_%s'arg arg))"
-      pvp proc.proc_symbol.xdr_name pvp;
-    fprintf f "@]@ @ ";
-
-    fprintf f "@[<hv 2>";
-    fprintf f "let %s'async client arg pass_reply =@ " proc.proc_symbol.ocaml_name;
-    (* fprintf f "assert(Rpc_client.program client == _program);@ "; *)
-    fprintf f "Rpc_client.add_call client \"%s\" (_of_%s'arg arg)@ "
-      proc.proc_symbol.xdr_name pvp;
-    fprintf f "  (fun g -> pass_reply (fun () -> _to_%s'res (g())))@ " pvp;
-    fprintf f "@]@ @ "
-
+    ( match inst with
+	| `Client ->
+	    ()
+	| `Make ->
+	    fprintf f "@[<hv 2>";
+	    fprintf f "let %s client arg =@ " proc.proc_symbol.ocaml_name;
+	    (* fprintf f "assert(Rpc_client.program client == _program);@ "; *)
+	    fprintf f "_to_%s'res (U'C.unbound_sync_call client _program \"%s\" (_of_%s'arg arg))"
+	      pvp proc.proc_symbol.xdr_name pvp;
+	    fprintf f "@]@ @ ";
+	    
+	    fprintf f "@[<hv 2>";
+	    fprintf f "let %s'async client arg pass_reply =@ " proc.proc_symbol.ocaml_name;
+	    (* fprintf f "assert(Rpc_client.program client == _program);@ "; *)
+	    fprintf f "U'C.unbound_async_call client _program \"%s\" (_of_%s'arg arg)@ "
+	      proc.proc_symbol.xdr_name pvp;
+	    fprintf f "  (fun g -> pass_reply (fun () -> _to_%s'res (g())))@ " pvp;
+	    fprintf f "@]@ @ "
+    )
   in
 
   fprintf mli "@[<v>";
